@@ -9,60 +9,12 @@
 #ifndef __I18N_LANGS_H__
 #define __I18N_LANGS_H__
 
+#include "I18N.h"
+
 namespace i2p
 {
 namespace i18n
 {
-	class Locale
-	{
-		public:
-			Locale (
-				const std::string& language,
-				const std::map<std::string, std::string>& strings,
-				const std::map<std::string, std::vector<std::string>>& plurals,
-				std::function<int(int)> formula
-			): m_Language (language), m_Strings (strings), m_Plurals (plurals), m_Formula (formula) { };
-
-			// Get activated language name for webconsole
-			std::string GetLanguage() const
-			{
-				return m_Language;
-			}
-
-			std::string GetString (const std::string& arg) const
-			{
-				const auto it = m_Strings.find(arg);
-				if (it == m_Strings.end())
-				{
-					return arg;
-				}
-				else
-				{
-					return it->second;
-				}
-			}
-
-			std::string GetPlural (const std::string& arg, const std::string& arg2, const int& n) const
-			{
-				const auto it = m_Plurals.find(arg2);
-				if (it == m_Plurals.end()) // not found, fallback to english
-				{
-					return n == 1 ? arg : arg2;
-				}
-				else
-				{
-					int form = m_Formula(n);
-					return it->second[form];
-				}
-			}
-
-		private:
-			const std::string m_Language;
-			const std::map<std::string, std::string> m_Strings;
-			const std::map<std::string, std::vector<std::string>> m_Plurals;
-			std::function<int(int)> m_Formula;
-	};
-
 	struct langData
 	{
 		std::string LocaleName; // localized name
@@ -74,10 +26,13 @@ namespace i18n
 	namespace afrikaans { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace armenian  { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace chinese   { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
+	namespace czech     { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace english   { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace french    { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace german    { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
+	namespace italian   { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace russian   { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
+	namespace spanish   { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace turkmen   { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace ukrainian { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
 	namespace uzbek     { std::shared_ptr<const i2p::i18n::Locale> GetLocale (); }
@@ -88,14 +43,17 @@ namespace i18n
 	static std::map<std::string, langData> languages
 	{
 		{ "afrikaans", {"Afrikaans", "af", i2p::i18n::afrikaans::GetLocale} },
-		{ "armenian", {"հայերէն", "hy", i2p::i18n::armenian::GetLocale} },
+		{ "armenian", {"hայերէն", "hy", i2p::i18n::armenian::GetLocale} },
 		{ "chinese", {"简体字", "zh-CN", i2p::i18n::chinese::GetLocale} },
+		{ "czech", {"čeština", "cs", i2p::i18n::czech::GetLocale} },
 		{ "english", {"English", "en", i2p::i18n::english::GetLocale} },
 		{ "french", {"Français", "fr", i2p::i18n::french::GetLocale} },
 		{ "german", {"Deutsch", "de", i2p::i18n::german::GetLocale} },
-		{ "russian", {"русский язык", "ru", i2p::i18n::russian::GetLocale} },
-		{ "turkmen", {"türkmen dili", "tk", i2p::i18n::turkmen::GetLocale} },
-		{ "ukrainian", {"украї́нська мо́ва", "uk", i2p::i18n::ukrainian::GetLocale} },
+		{ "italian", {"Italiano", "it", i2p::i18n::italian::GetLocale} },
+		{ "russian", {"Русский язык", "ru", i2p::i18n::russian::GetLocale} },
+		{ "spanish", {"Español", "es", i2p::i18n::spanish::GetLocale} },
+		{ "turkmen", {"Türkmen dili", "tk", i2p::i18n::turkmen::GetLocale} },
+		{ "ukrainian", {"Украї́нська мо́ва", "uk", i2p::i18n::ukrainian::GetLocale} },
 		{ "uzbek", {"Oʻzbek", "uz", i2p::i18n::uzbek::GetLocale} },
 	};
 
