@@ -17,14 +17,14 @@
 namespace i2p
 {
 namespace transport
-{	
+{
 	SSU2Server::SSU2Server ():
 		RunnableServiceWithWork ("SSU2"), m_ReceiveService ("SSU2r"),
 		m_SocketV4 (m_ReceiveService.GetService ()), m_SocketV6 (m_ReceiveService.GetService ()),
 		m_AddressV4 (boost::asio::ip::address_v4()), m_AddressV6 (boost::asio::ip::address_v6()),
-		m_TerminationTimer (GetService ()), m_ResendTimer (GetService ()), 
+		m_TerminationTimer (GetService ()), m_ResendTimer (GetService ()),
 		m_IntroducersUpdateTimer (GetService ()), m_IntroducersUpdateTimerV6 (GetService ()),
-		m_IsPublished (true), m_IsSyncClockFromPeers (true)
+		m_IsPublished (true), m_IsSyncClockFromPeers (true), m_IsThroughProxy (false)
 	{
 	}
 
@@ -36,12 +36,32 @@ namespace transport
 			i2p::config::GetOption ("ssu2.published", m_IsPublished);
 			i2p::config::GetOption("nettime.frompeers", m_IsSyncClockFromPeers);
 			bool found = false;
-			auto& addresses = i2p::context.GetRouterInfo ().GetAddresses ();
-			for (const auto& address: addresses)
+			auto addresses = i2p::context.GetRouterInfo ().GetAddresses ();
+			if (!addresses) return;
+			for (const auto& address: *addresses)
 			{
 				if (!address) continue;
 				if (address->transportStyle == i2p::data::RouterInfo::eTransportSSU2)
 				{
+					if (m_IsThroughProxy)
+					{
+						found = true;
+						if (address->IsV6 ())
+						{
+							uint16_t mtu; i2p::config::GetOption ("ssu2.mtu6", mtu);
+							if (!mtu || mtu > SSU2_MAX_PACKET_SIZE - SOCKS5_UDP_IPV6_REQUEST_HEADER_SIZE)
+								mtu = SSU2_MAX_PACKET_SIZE - SOCKS5_UDP_IPV6_REQUEST_HEADER_SIZE;
+							i2p::context.SetMTU (mtu, false);
+						}
+						else
+						{
+							uint16_t mtu; i2p::config::GetOption ("ssu2.mtu4", mtu);
+							if (!mtu || mtu > SSU2_MAX_PACKET_SIZE - SOCKS5_UDP_IPV4_REQUEST_HEADER_SIZE)
+								mtu = SSU2_MAX_PACKET_SIZE - SOCKS5_UDP_IPV4_REQUEST_HEADER_SIZE;
+							i2p::context.SetMTU (mtu, true);
+						}
+						continue; // we don't need port for proxy
+					}
 					auto port = address->port;
 					if (!port)
 					{
@@ -49,9 +69,8 @@ namespace transport
 						if (ssu2Port) port = ssu2Port;
 						else
 						{
-							bool ssu;   i2p::config::GetOption("ssu", ssu);
 							uint16_t p; i2p::config::GetOption ("port", p);
-							if (p) port = ssu ? (p + 1) : p;
+							if (p) port = p;
 						}
 					}
 					if (port)
@@ -84,33 +103,44 @@ namespace transport
 				}
 			}
 			if (found)
+			{
+				if (m_IsThroughProxy)
+					ConnectToProxy ();
 				m_ReceiveService.Start ();
+			}
 			ScheduleTermination ();
+			ScheduleResend (false);
 		}
 	}
 
 	void SSU2Server::Stop ()
 	{
 		if (IsRunning ())
-		{	
+		{
 			m_TerminationTimer.cancel ();
 			m_ResendTimer.cancel ();
 			m_IntroducersUpdateTimer.cancel ();
 			m_IntroducersUpdateTimerV6.cancel ();
-		}	
-		
+		}
+
 		auto sessions = m_Sessions;
 		for (auto& it: sessions)
-		{	
+		{
 			it.second->RequestTermination (eSSU2TerminationReasonRouterShutdown);
 			it.second->Done ();
-		}	
-		
+		}
+
 		if (context.SupportsV4 () || context.SupportsV6 ())
 			m_ReceiveService.Stop ();
 		m_SocketV4.close ();
 		m_SocketV6.close ();
-		
+
+		if (m_UDPAssociateSocket)
+		{
+			m_UDPAssociateSocket->close ();
+			m_UDPAssociateSocket.reset (nullptr);
+		}
+
 		StopIOService ();
 
 		m_Sessions.clear ();
@@ -125,47 +155,56 @@ namespace transport
 	{
 		if (localAddress.is_unspecified ()) return;
 		if (localAddress.is_v4 ())
-		{	
+		{
 			m_AddressV4 = localAddress;
-			int mtu = i2p::util::net::GetMTU (localAddress);
+			uint16_t mtu; i2p::config::GetOption ("ssu2.mtu4", mtu);
+			if (!mtu) mtu = i2p::util::net::GetMTU (localAddress);
 			if (mtu < (int)SSU2_MIN_PACKET_SIZE) mtu = SSU2_MIN_PACKET_SIZE;
 			if (mtu > (int)SSU2_MAX_PACKET_SIZE) mtu = SSU2_MAX_PACKET_SIZE;
 			i2p::context.SetMTU (mtu, true);
-		}	
+		}
 		else if (localAddress.is_v6 ())
-		{	
+		{
 			m_AddressV6 = localAddress;
-			int maxMTU = i2p::util::net::GetMaxMTU (localAddress.to_v6 ());
-			int mtu = i2p::util::net::GetMTU (localAddress);
-			if (mtu > maxMTU) mtu = maxMTU;
+			uint16_t mtu; i2p::config::GetOption ("ssu2.mtu6", mtu);
+			if (!mtu)
+			{
+				int maxMTU = i2p::util::net::GetMaxMTU (localAddress.to_v6 ());
+				mtu = i2p::util::net::GetMTU (localAddress);
+				if (mtu > maxMTU) mtu = maxMTU;
+			}
+			else
+				if (mtu > (int)SSU2_MAX_PACKET_SIZE) mtu = SSU2_MAX_PACKET_SIZE;
 			if (mtu < (int)SSU2_MIN_PACKET_SIZE) mtu = SSU2_MIN_PACKET_SIZE;
 			i2p::context.SetMTU (mtu, false);
-		}	
-	}	
+		}
+	}
 
 	bool SSU2Server::IsSupported (const boost::asio::ip::address& addr) const
 	{
+		if (m_IsThroughProxy)
+			return m_SocketV4.is_open ();
 		if (addr.is_v4 ())
-		{	
-			if (m_SocketV4.is_open ()) 
+		{
+			if (m_SocketV4.is_open ())
 				return true;
-		}	
+		}
 		else if (addr.is_v6 ())
-		{	
-			if (m_SocketV6.is_open ()) 
+		{
+			if (m_SocketV6.is_open ())
 				return true;
 		}
 		return false;
-	}	
+	}
 
 	uint16_t SSU2Server::GetPort (bool v4) const
 	{
 		boost::system::error_code ec;
-		boost::asio::ip::udp::endpoint ep = v4 ? m_SocketV4.local_endpoint (ec) : m_SocketV6.local_endpoint (ec);
+		boost::asio::ip::udp::endpoint ep = (v4 || m_IsThroughProxy) ? m_SocketV4.local_endpoint (ec) : m_SocketV6.local_endpoint (ec);
 		if (ec) return 0;
 		return ep.port ();
-	}	
-		
+	}
+
 	boost::asio::ip::udp::socket& SSU2Server::OpenSocket (const boost::asio::ip::udp::endpoint& localEndpoint)
 	{
 		boost::asio::ip::udp::socket& socket = localEndpoint.address ().is_v6 () ? m_SocketV6 : m_SocketV4;
@@ -197,7 +236,19 @@ namespace transport
 	void SSU2Server::HandleReceivedFrom (const boost::system::error_code& ecode, size_t bytes_transferred,
 		Packet * packet, boost::asio::ip::udp::socket& socket)
 	{
-		if (!ecode)
+		if (!ecode
+			|| ecode == boost::asio::error::connection_refused
+			|| ecode == boost::asio::error::connection_reset
+			|| ecode == boost::asio::error::network_unreachable
+			|| ecode == boost::asio::error::host_unreachable
+#ifdef _WIN32 // windows can throw WinAPI error, which is not handled by ASIO
+			|| ecode.value() == boost::winapi::ERROR_CONNECTION_REFUSED_
+			|| ecode.value() == boost::winapi::ERROR_NETWORK_UNREACHABLE_
+			|| ecode.value() == boost::winapi::ERROR_HOST_UNREACHABLE_
+#endif
+		)
+		// just try continue reading when received ICMP response otherwise socket can crash,
+		// but better to find out which host were sent it and mark that router as unreachable
 		{
 			i2p::transport::transports.UpdateReceivedBytes (bytes_transferred);
 			packet->len = bytes_transferred;
@@ -238,10 +289,20 @@ namespace transport
 			if (ecode != boost::asio::error::operation_aborted)
 			{
 				LogPrint (eLogError, "SSU2: Receive error: code ", ecode.value(), ": ", ecode.message ());
-				auto ep = socket.local_endpoint ();
-				socket.close ();
-				OpenSocket (ep);
-				Receive (socket);
+				if (m_IsThroughProxy)
+				{
+					m_UDPAssociateSocket.reset (nullptr);
+					m_ProxyRelayEndpoint.reset (nullptr);
+					m_SocketV4.close ();
+					ConnectToProxy ();
+				}
+				else
+				{
+					auto ep = socket.local_endpoint ();
+					socket.close ();
+					OpenSocket (ep);
+					Receive (socket);
+				}
 			}
 		}
 	}
@@ -250,19 +311,26 @@ namespace transport
 	{
 		if (packet)
 		{
-			ProcessNextPacket (packet->buf, packet->len, packet->from);
+			if (m_IsThroughProxy)
+				ProcessNextPacketFromProxy (packet->buf, packet->len);
+			else
+				ProcessNextPacket (packet->buf, packet->len, packet->from);
 			m_PacketsPool.ReleaseMt (packet);
-			if (m_LastSession && m_LastSession->GetState () != eSSU2SessionStateTerminated) 
+			if (m_LastSession && m_LastSession->GetState () != eSSU2SessionStateTerminated)
 				m_LastSession->FlushData ();
 		}
 	}
 
 	void SSU2Server::HandleReceivedPackets (std::vector<Packet *> packets)
 	{
-		for (auto& packet: packets)
-			ProcessNextPacket (packet->buf, packet->len, packet->from);
+		if (m_IsThroughProxy)
+			for (auto& packet: packets)
+				ProcessNextPacketFromProxy (packet->buf, packet->len);
+		else
+			for (auto& packet: packets)
+				ProcessNextPacket (packet->buf, packet->len, packet->from);
 		m_PacketsPool.ReleaseMt (packets);
-		if (m_LastSession && m_LastSession->GetState () != eSSU2SessionStateTerminated) 
+		if (m_LastSession && m_LastSession->GetState () != eSSU2SessionStateTerminated)
 			m_LastSession->FlushData ();
 	}
 
@@ -288,7 +356,7 @@ namespace transport
 			m_Sessions.erase (it);
 		}
 	}
-		
+
 	void SSU2Server::AddSessionByRouterHash (std::shared_ptr<SSU2Session> session)
 	{
 		if (session)
@@ -313,6 +381,7 @@ namespace transport
 	bool SSU2Server::AddPendingOutgoingSession (std::shared_ptr<SSU2Session> session)
 	{
 		if (!session) return false;
+		std::unique_lock<std::mutex> l(m_PendingOutgoingSessionsMutex);
 		return m_PendingOutgoingSessions.emplace (session->GetRemoteEndpoint (), session).second;
 	}
 
@@ -322,10 +391,11 @@ namespace transport
 		if (it != m_SessionsByRouterHash.end ())
 			return it->second;
 		return nullptr;
-	}	
+	}
 
 	std::shared_ptr<SSU2Session> SSU2Server::FindPendingOutgoingSession (const boost::asio::ip::udp::endpoint& ep) const
-	{		
+	{
+		std::unique_lock<std::mutex> l(m_PendingOutgoingSessionsMutex);
 		auto it = m_PendingOutgoingSessions.find (ep);
 		if (it != m_PendingOutgoingSessions.end ())
 			return it->second;
@@ -334,9 +404,10 @@ namespace transport
 
 	void SSU2Server::RemovePendingOutgoingSession (const boost::asio::ip::udp::endpoint& ep)
 	{
+		std::unique_lock<std::mutex> l(m_PendingOutgoingSessionsMutex);
 		m_PendingOutgoingSessions.erase (ep);
-	}	
-		
+	}
+
 	std::shared_ptr<SSU2Session> SSU2Server::GetRandomSession (
 		i2p::data::RouterInfo::CompatibleTransports remoteTransports, const i2p::data::IdentHash& excluded) const
 	{
@@ -348,23 +419,23 @@ namespace transport
 		std::advance (it, ind);
 		while (it != m_Sessions.end ())
 		{
-			if ((it->second->GetRemoteTransports () & remoteTransports) && 
+			if ((it->second->GetRemoteTransports () & remoteTransports) &&
 			    it->second->GetRemoteIdentity ()->GetIdentHash () != excluded)
 				return it->second;
 			it++;
 		}
-		// not found, try from begining
+		// not found, try from beginning
 		it = m_Sessions.begin ();
 		while (it != m_Sessions.end () && ind)
 		{
-			if ((it->second->GetRemoteTransports () & remoteTransports) && 
+			if ((it->second->GetRemoteTransports () & remoteTransports) &&
 			    it->second->GetRemoteIdentity ()->GetIdentHash () != excluded)
 				return it->second;
 			it++; ind--;
-		}	
+		}
 		return nullptr;
-	}	
-		
+	}
+
 	void SSU2Server::AddRelay (uint32_t tag, std::shared_ptr<SSU2Session> relay)
 	{
 		m_Relays.emplace (tag, relay);
@@ -408,41 +479,42 @@ namespace transport
 			switch (m_LastSession->GetState ())
 			{
 				case eSSU2SessionStateEstablished:
-				case eSSU2SessionStateSessionConfirmedSent:	
-					m_LastSession->ProcessData (buf, len);
+				case eSSU2SessionStateSessionConfirmedSent:
+					m_LastSession->ProcessData (buf, len, senderEndpoint);
 				break;
 				case eSSU2SessionStateSessionCreatedSent:
 					if (!m_LastSession->ProcessSessionConfirmed (buf, len))
 					{
 						m_LastSession->Done ();
 						m_LastSession = nullptr;
-					}	
+					}
 				break;
 				case eSSU2SessionStateIntroduced:
-					if (m_LastSession->GetRemoteEndpoint ().address ().is_unspecified ())	
+					if (m_LastSession->GetRemoteEndpoint ().address ().is_unspecified ())
 						m_LastSession->SetRemoteEndpoint (senderEndpoint);
-					if (m_LastSession->GetRemoteEndpoint () == senderEndpoint)
+					if (m_LastSession->GetRemoteEndpoint ().address () == senderEndpoint.address ()) // port might be different
 						m_LastSession->ProcessHolePunch (buf, len);
 					else
 					{
-						LogPrint (eLogWarning, "SSU2: HolePunch endpoint ", senderEndpoint,
-							" doesn't match RelayResponse ", m_LastSession->GetRemoteEndpoint ());
+						LogPrint (eLogWarning, "SSU2: HolePunch address ", senderEndpoint.address (),
+							" doesn't match RelayResponse ", m_LastSession->GetRemoteEndpoint ().address ());
 						m_LastSession->Done ();
-						m_LastSession = nullptr;	
-					}		
+						m_LastSession = nullptr;
+					}
 				break;
 				case eSSU2SessionStatePeerTest:
 					m_LastSession->SetRemoteEndpoint (senderEndpoint);
 					m_LastSession->ProcessPeerTest (buf, len);
 				break;
 				case eSSU2SessionStateClosing:
-					m_LastSession->ProcessData (buf, len); // we might receive termintaion block
-					if (m_LastSession && m_LastSession->GetState () != eSSU2SessionStateTerminated)
+					m_LastSession->ProcessData (buf, len, senderEndpoint); // we might receive termintaion block
+					if (m_LastSession && m_LastSession->GetState () == eSSU2SessionStateClosing)
 						m_LastSession->RequestTermination (eSSU2TerminationReasonIdleTimeout); // send termination again
-				break;	
+				break;
+				case eSSU2SessionStateClosingConfirmed:
 				case eSSU2SessionStateTerminated:
 					m_LastSession = nullptr;
-				break;	
+				break;
 				default:
 					LogPrint (eLogWarning, "SSU2: Invalid session state ", (int)m_LastSession->GetState ());
 			}
@@ -455,23 +527,33 @@ namespace transport
 			{
 				if (it1->second->GetState () == eSSU2SessionStateSessionRequestSent &&
 					it1->second->ProcessSessionCreated (buf, len))
-					m_PendingOutgoingSessions.erase (it1); // we are done with that endpoint			
+				{
+					std::unique_lock<std::mutex> l(m_PendingOutgoingSessionsMutex);
+					m_PendingOutgoingSessions.erase (it1); // we are done with that endpoint
+				}
 				else
 					it1->second->ProcessRetry (buf, len);
 			}
-			else
+			else if (!i2p::util::net::IsInReservedRange(senderEndpoint.address ()) && senderEndpoint.port ())
 			{
 				// assume new incoming session
 				auto session = std::make_shared<SSU2Session> (*this);
 				session->SetRemoteEndpoint (senderEndpoint);
 				session->ProcessFirstIncomingMessage (connID, buf, len);
 			}
+			else
+				LogPrint (eLogError, "SSU2: Incoming packet received from invalid endpoint ", senderEndpoint);
 		}
 	}
 
 	void SSU2Server::Send (const uint8_t * header, size_t headerLen, const uint8_t * payload, size_t payloadLen,
 		const boost::asio::ip::udp::endpoint& to)
 	{
+		if (m_IsThroughProxy)
+		{
+			SendThroughProxy (header, headerLen, nullptr, 0, payload, payloadLen, to);
+			return;
+		}
 		std::vector<boost::asio::const_buffer> bufs
 		{
 			boost::asio::buffer (header, headerLen),
@@ -491,6 +573,11 @@ namespace transport
 	void SSU2Server::Send (const uint8_t * header, size_t headerLen, const uint8_t * headerX, size_t headerXLen,
 		const uint8_t * payload, size_t payloadLen, const boost::asio::ip::udp::endpoint& to)
 	{
+		if (m_IsThroughProxy)
+		{
+			SendThroughProxy (header, headerLen, headerX, headerXLen, payload, payloadLen, to);
+			return;
+		}
 		std::vector<boost::asio::const_buffer> bufs
 		{
 			boost::asio::buffer (header, headerLen),
@@ -523,41 +610,41 @@ namespace transport
 				{
 					auto session = it->second;
 					GetService ().post ([session]() { session->SendPeerTest (); });
-				}	
+				}
 				return false;
-			}	
+			}
 			// check is no pending session
 			bool isValidEndpoint = !address->host.is_unspecified () && address->port;
 			if (isValidEndpoint)
-			{	
+			{
 				if (i2p::util::net::IsInReservedRange(address->host)) return false;
 				auto s = FindPendingOutgoingSession (boost::asio::ip::udp::endpoint (address->host, address->port));
 				if (s)
-				{	
+				{
 					if (peerTest)
 					{
 						// if peer test requested add it to the list for pending session
 						auto onEstablished = s->GetOnEstablished ();
 						if (onEstablished)
-							s->SetOnEstablished ([s, onEstablished]() 
-								{ 
-									onEstablished (); 
-									s->SendPeerTest (); 
-								 });
-						else	
+							s->SetOnEstablished ([s, onEstablished]()
+								{
+									onEstablished ();
+									s->SendPeerTest ();
+								});
+						else
 							s->SetOnEstablished ([s]() { s->SendPeerTest (); });
-					}	
+					}
 					return false;
-				}	
-			}	
-			
+				}
+			}
+
 			auto session = std::make_shared<SSU2Session> (*this, router, address);
 			if (peerTest)
 				session->SetOnEstablished ([session]() {session->SendPeerTest (); });
 
 			if (address->UsesIntroducer ())
 				GetService ().post (std::bind (&SSU2Server::ConnectThroughIntroducer, this, session));
-			else if (isValidEndpoint) // we can't connect without endpoint				
+			else if (isValidEndpoint) // we can't connect without endpoint
 				GetService ().post ([session]() { session->Connect (); });
 			else
 				return false;
@@ -576,7 +663,7 @@ namespace transport
 		// try to find existing session first
 		for (auto& it: address->ssu->introducers)
 		{
-			auto it1 = m_SessionsByRouterHash.find (it.iKey);
+			auto it1 = m_SessionsByRouterHash.find (it.iH);
 			if (it1 != m_SessionsByRouterHash.end ())
 			{
 				it1->second->Introduce (session, it.iTag);
@@ -588,26 +675,26 @@ namespace transport
 		std::shared_ptr<i2p::data::RouterInfo> r;
 		uint32_t relayTag = 0;
 		if (!address->ssu->introducers.empty ())
-		{	
-			std::vector<int> indicies;
-			for (int i = 0; i < (int)address->ssu->introducers.size (); i++) indicies.push_back(i);
-			if (indicies.size () > 1)
-				std::shuffle (indicies.begin(), indicies.end(), std::mt19937(std::random_device()()));
+		{
+			std::vector<int> indices;
+			for (int i = 0; i < (int)address->ssu->introducers.size (); i++) indices.push_back(i);
+			if (indices.size () > 1)
+				std::shuffle (indices.begin(), indices.end(), std::mt19937(std::random_device()()));
 
-			for (auto i: indicies)
+			for (auto i: indices)
 			{
-				const auto& introducer = address->ssu->introducers[indicies[i]]; 
+				const auto& introducer = address->ssu->introducers[indices[i]];
 				if (introducer.iTag && ts < introducer.iExp)
-				{	
-					r = i2p::data::netdb.FindRouter (introducer.iKey);
+				{
+					r = i2p::data::netdb.FindRouter (introducer.iH);
 					if (r && r->IsReachableFrom (i2p::context.GetRouterInfo ()))
 					{
 						relayTag = introducer.iTag;
 						if (relayTag) break;
 					}
-				}	
+				}
 			}
-		}	
+		}
 		if (r)
 		{
 			if (relayTag)
@@ -619,10 +706,10 @@ namespace transport
 					bool isValidEndpoint = !addr->host.is_unspecified () && addr->port &&
 						!i2p::util::net::IsInReservedRange(addr->host);
 					if (isValidEndpoint)
-					{	
+					{
 						auto s = FindPendingOutgoingSession (boost::asio::ip::udp::endpoint (addr->host, addr->port));
 						if (!s)
-						{	
+						{
 							s = std::make_shared<SSU2Session> (*this, r, addr);
 							s->SetOnEstablished ([session, s, relayTag]() { s->Introduce (session, relayTag); });
 							s->Connect ();
@@ -636,10 +723,10 @@ namespace transport
 										onEstablished ();
 										s->Introduce (session, relayTag);
 									});
-							else	
+							else
 								s->SetOnEstablished ([session, s, relayTag]() {s->Introduce (session, relayTag); });
-						}	
-					}	
+						}
+					}
 				}
 			}
 		}
@@ -648,7 +735,7 @@ namespace transport
 			// introducers not found, try to request them
 			for (auto& it: address->ssu->introducers)
 				if (it.iTag && ts < it.iExp)
-					i2p::data::netdb.RequestDestination (it.iKey);
+					i2p::data::netdb.RequestDestination (it.iH);
 		}
 	}
 
@@ -663,15 +750,15 @@ namespace transport
 			auto s = it->second;
 			if (it->second->IsEstablished ())
 				GetService ().post ([s]() { s->SendPeerTest (); });
-			else	
-				s->SetOnEstablished ([s]() { s->SendPeerTest (); });	
-			return true;	
+			else
+				s->SetOnEstablished ([s]() { s->SendPeerTest (); });
+			return true;
 		}
-		else 
+		else
 			CreateSession (router, addr, true);
 		return true;
-	}	
-		
+	}
+
 	void SSU2Server::ScheduleTermination ()
 	{
 		m_TerminationTimer.expires_from_now (boost::posix_time::seconds(SSU2_TERMINATION_CHECK_TIMEOUT));
@@ -689,6 +776,7 @@ namespace transport
 				if (it->second->IsTerminationTimeoutExpired (ts))
 				{
 					//it->second->Terminate ();
+					std::unique_lock<std::mutex> l(m_PendingOutgoingSessionsMutex);
 					it = m_PendingOutgoingSessions.erase (it);
 				}
 				else
@@ -717,7 +805,7 @@ namespace transport
 					it = m_SessionsByRouterHash.erase (it);
 				else
 					it++;
-			}		
+			}
 
 			for (auto it = m_Relays.begin (); it != m_Relays.begin ();)
 			{
@@ -725,8 +813,8 @@ namespace transport
 					it = m_Relays.erase (it);
 				else
 					it++;
-			}	
-			
+			}
+
 			for (auto it = m_IncomingTokens.begin (); it != m_IncomingTokens.end (); )
 			{
 				if (ts > it->second.second)
@@ -742,16 +830,17 @@ namespace transport
 				else
 					it++;
 			}
-			
+
 			m_PacketsPool.CleanUpMt ();
 			m_SentPacketsPool.CleanUp ();
 			ScheduleTermination ();
 		}
 	}
 
-	void SSU2Server::ScheduleResend ()
+	void SSU2Server::ScheduleResend (bool more)
 	{
-		m_ResendTimer.expires_from_now (boost::posix_time::milliseconds(SSU2_RESEND_CHECK_TIMEOUT));
+		m_ResendTimer.expires_from_now (boost::posix_time::milliseconds (more ? SSU2_RESEND_CHECK_MORE_TIMEOUT :
+			(SSU2_RESEND_CHECK_TIMEOUT + rand () % SSU2_RESEND_CHECK_TIMEOUT_VARIANCE)));
 		m_ResendTimer.async_wait (std::bind (&SSU2Server::HandleResendTimer,
 			this, std::placeholders::_1));
 	}
@@ -760,12 +849,16 @@ namespace transport
 	{
 		if (ecode != boost::asio::error::operation_aborted)
 		{
+			size_t resentPacketsNum = 0;
 			auto ts = i2p::util::GetMillisecondsSinceEpoch ();
 			for (auto it: m_Sessions)
-				it.second->Resend (ts);
+			{
+				resentPacketsNum += it.second->Resend (ts);
+				if (resentPacketsNum > SSU2_MAX_RESEND_PACKETS) break;
+			}
 			for (auto it: m_PendingOutgoingSessions)
 				it.second->Resend (ts);
-			ScheduleResend ();
+			ScheduleResend (resentPacketsNum > SSU2_MAX_RESEND_PACKETS);
 		}
 	}
 
@@ -774,26 +867,36 @@ namespace transport
 		m_OutgoingTokens[ep] = {token, exp};
 	}
 
-	uint64_t SSU2Server::FindOutgoingToken (const boost::asio::ip::udp::endpoint& ep) const
+	uint64_t SSU2Server::FindOutgoingToken (const boost::asio::ip::udp::endpoint& ep)
 	{
 		auto it = m_OutgoingTokens.find (ep);
 		if (it != m_OutgoingTokens.end ())
 		{
 			if (i2p::util::GetSecondsSinceEpoch () + SSU2_TOKEN_EXPIRATION_THRESHOLD > it->second.second)
-				return 0; // token expired
+			{
+				// token expired
+				m_OutgoingTokens.erase (it);
+				return 0;
+			}
 			return it->second.first;
-		}	
+		}
 		return 0;
 	}
 
 	uint64_t SSU2Server::GetIncomingToken (const boost::asio::ip::udp::endpoint& ep)
 	{
+		auto ts = i2p::util::GetSecondsSinceEpoch ();
 		auto it = m_IncomingTokens.find (ep);
 		if (it != m_IncomingTokens.end ())
-			return it->second.first;
+		{
+			if (ts + SSU2_TOKEN_EXPIRATION_THRESHOLD <= it->second.second)
+				return it->second.first;
+			else // token expired
+				m_IncomingTokens.erase (it);
+		}
 		uint64_t token;
 		RAND_bytes ((uint8_t *)&token, 8);
-		m_IncomingTokens.emplace (ep, std::make_pair (token, i2p::util::GetSecondsSinceEpoch () + SSU2_TOKEN_EXPIRATION_TIMEOUT));
+		m_IncomingTokens.emplace (ep, std::make_pair (token, ts + SSU2_TOKEN_EXPIRATION_TIMEOUT));
 		return token;
 	}
 
@@ -802,23 +905,23 @@ namespace transport
 		m_IncomingTokens.erase (ep); // drop previous
 		uint64_t token;
 		RAND_bytes ((uint8_t *)&token, 8);
-		auto ret = std::make_pair (token, i2p::util::GetSecondsSinceEpoch () + SSU2_NEXT_TOKEN_EXPIRATION_TIMEOUT); 
+		auto ret = std::make_pair (token, i2p::util::GetSecondsSinceEpoch () + SSU2_NEXT_TOKEN_EXPIRATION_TIMEOUT);
 		m_IncomingTokens.emplace (ep, ret);
 		return ret;
-	}	
+	}
 
-	std::list<std::shared_ptr<SSU2Session> > SSU2Server::FindIntroducers (int maxNumIntroducers, 
+	std::list<std::shared_ptr<SSU2Session> > SSU2Server::FindIntroducers (int maxNumIntroducers,
 		bool v4, const std::set<i2p::data::IdentHash>& excluded) const
 	{
 		std::list<std::shared_ptr<SSU2Session> > ret;
 		for (const auto& s : m_Sessions)
 		{
-			if (s.second->IsEstablished () && (s.second->GetRelayTag () && s.second->IsOutgoing ()) &&	
+			if (s.second->IsEstablished () && (s.second->GetRelayTag () && s.second->IsOutgoing ()) &&
 			    !excluded.count (s.second->GetRemoteIdentity ()->GetIdentHash ()) &&
 			    ((v4 && (s.second->GetRemoteTransports () & i2p::data::RouterInfo::eSSU2V4)) ||
-			     (!v4 && (s.second->GetRemoteTransports () & i2p::data::RouterInfo::eSSU2V6))))
+			    (!v4 && (s.second->GetRemoteTransports () & i2p::data::RouterInfo::eSSU2V6))))
 				ret.push_back (s.second);
-		}	
+		}
 		if ((int)ret.size () > maxNumIntroducers)
 		{
 			// shink ret randomly
@@ -832,7 +935,7 @@ namespace transport
 			}
 		}
 		return ret;
-	}	
+	}
 
 	void SSU2Server::UpdateIntroducers (bool v4)
 	{
@@ -844,11 +947,11 @@ namespace transport
 		{
 			std::shared_ptr<SSU2Session> session;
 			auto it1 = m_SessionsByRouterHash.find (it);
-			if (it1 != m_SessionsByRouterHash.end ()) 
-			{	
+			if (it1 != m_SessionsByRouterHash.end ())
+			{
 				session = it1->second;
 				excluded.insert (it);
-			}	
+			}
 			if (session && session->IsEstablished ())
 			{
 				if (ts < session->GetCreationTime () + SSU2_TO_INTRODUCER_SESSION_EXPIRATION)
@@ -857,10 +960,10 @@ namespace transport
 					newList.push_back (it);
 				else
 					session = nullptr;
-			}	
+			}
 			if (!session)
-				i2p::context.RemoveSSU2Introducer (it, v4);	
-		}	
+				i2p::context.RemoveSSU2Introducer (it, v4);
+		}
 		if (newList.size () < SSU2_MAX_NUM_INTRODUCERS)
 		{
 			auto sessions = FindIntroducers (SSU2_MAX_NUM_INTRODUCERS - newList.size (), v4, excluded);
@@ -869,40 +972,41 @@ namespace transport
 				// bump creation time for previous introducers if no new sessions found
 				LogPrint (eLogDebug, "SSU2: No new introducers found. Trying to reuse existing");
 				for (auto& it : introducers)
-				{	
+				{
 					auto it1 = m_SessionsByRouterHash.find (it);
 					if (it1 != m_SessionsByRouterHash.end ())
 					{
 						auto session = it1->second;
 						if (session->IsEstablished ())
-						{	
+						{
 							session->SetCreationTime (session->GetCreationTime () + SSU2_TO_INTRODUCER_SESSION_DURATION);
-							if (std::find (newList.begin (), newList.end (), it) == newList.end ()) 
-							{	
+							if (std::find (newList.begin (), newList.end (), it) == newList.end ())
+							{
 								newList.push_back (it);
 								sessions.push_back (session);
-							}	
-						}	
-					}	
-				}	
-			}	
-						
+							}
+						}
+					}
+				}
+			}
+
 			for (const auto& it : sessions)
 			{
 				i2p::data::RouterInfo::Introducer introducer;
 				introducer.iTag = it->GetRelayTag ();
-				introducer.iKey = it->GetRemoteIdentity ()->GetIdentHash ();
+				introducer.iH = it->GetRemoteIdentity ()->GetIdentHash ();
+				introducer.isH = true;
 				introducer.iExp = it->GetCreationTime () + SSU2_TO_INTRODUCER_SESSION_EXPIRATION;
 				excluded.insert (it->GetRemoteIdentity ()->GetIdentHash ());
 				if (i2p::context.AddSSU2Introducer (introducer, v4))
 				{
-					LogPrint (eLogDebug, "SSU2: Introducer added ", it->GetRelayTag (), " at ", 
+					LogPrint (eLogDebug, "SSU2: Introducer added ", it->GetRelayTag (), " at ",
 						i2p::data::GetIdentHashAbbreviation (it->GetRemoteIdentity ()->GetIdentHash ()));
 					newList.push_back (it->GetRemoteIdentity ()->GetIdentHash ());
 					if (newList.size () >= SSU2_MAX_NUM_INTRODUCERS) break;
-				}	
-			}	
-		}	
+				}
+			}
+		}
 		introducers = newList;
 
 		if (introducers.size () < SSU2_MAX_NUM_INTRODUCERS)
@@ -931,12 +1035,12 @@ namespace transport
 	void SSU2Server::ScheduleIntroducersUpdateTimer ()
 	{
 		if (m_IsPublished)
-		{	
+		{
 			m_IntroducersUpdateTimer.expires_from_now (boost::posix_time::seconds(SSU2_KEEP_ALIVE_INTERVAL));
 			m_IntroducersUpdateTimer.async_wait (std::bind (&SSU2Server::HandleIntroducersUpdateTimer,
 				this, std::placeholders::_1, true));
-		}	
-	}	
+		}
+	}
 
 	void SSU2Server::RescheduleIntroducersUpdateTimer ()
 	{
@@ -948,18 +1052,18 @@ namespace transport
 			m_IntroducersUpdateTimer.expires_from_now (boost::posix_time::seconds(SSU2_KEEP_ALIVE_INTERVAL/2));
 			m_IntroducersUpdateTimer.async_wait (std::bind (&SSU2Server::HandleIntroducersUpdateTimer,
 				this, std::placeholders::_1, true));
-		}	
+		}
 	}
 
 	void SSU2Server::ScheduleIntroducersUpdateTimerV6 ()
 	{
 		if (m_IsPublished)
-		{	
+		{
 			m_IntroducersUpdateTimerV6.expires_from_now (boost::posix_time::seconds(SSU2_KEEP_ALIVE_INTERVAL));
 			m_IntroducersUpdateTimerV6.async_wait (std::bind (&SSU2Server::HandleIntroducersUpdateTimer,
 				this, std::placeholders::_1, false));
-		}	
-	}	
+		}
+	}
 
 	void SSU2Server::RescheduleIntroducersUpdateTimerV6 ()
 	{
@@ -971,9 +1075,9 @@ namespace transport
 			m_IntroducersUpdateTimerV6.expires_from_now (boost::posix_time::seconds(SSU2_KEEP_ALIVE_INTERVAL/2));
 			m_IntroducersUpdateTimerV6.async_wait (std::bind (&SSU2Server::HandleIntroducersUpdateTimer,
 				this, std::placeholders::_1, false));
-		}	
+		}
 	}
-		
+
 	void SSU2Server::HandleIntroducersUpdateTimer (const boost::system::error_code& ecode, bool v4)
 	{
 		if (ecode != boost::asio::error::operation_aborted)
@@ -997,8 +1101,8 @@ namespace transport
 				// we are firewalled
 				auto addr = i2p::context.GetRouterInfo ().GetSSU2V4Address ();
 				if (addr && addr->ssu && addr->ssu->introducers.empty ())
-					i2p::context.SetUnreachableSSU2 (true, false); // v4
-				
+					i2p::context.SetUnreachable (true, false); // v4
+
 				UpdateIntroducers (true);
 				ScheduleIntroducersUpdateTimer ();
 			}
@@ -1020,12 +1124,275 @@ namespace transport
 				// we are firewalled
 				auto addr = i2p::context.GetRouterInfo ().GetSSU2V6Address ();
 				if (addr && addr->ssu && addr->ssu->introducers.empty ())
-					i2p::context.SetUnreachableSSU2 (false, true); // v6
-				
+					i2p::context.SetUnreachable (false, true); // v6
+
 				UpdateIntroducers (false);
 				ScheduleIntroducersUpdateTimerV6 ();
-			}	
+			}
 		}
-	}	
+	}
+
+	void SSU2Server::SendThroughProxy (const uint8_t * header, size_t headerLen, const uint8_t * headerX, size_t headerXLen,
+		const uint8_t * payload, size_t payloadLen, const boost::asio::ip::udp::endpoint& to)
+	{
+		if (!m_ProxyRelayEndpoint) return;
+		size_t requestHeaderSize = 0;
+		memset (m_UDPRequestHeader, 0, 3);
+		if (to.address ().is_v6 ())
+		{
+			m_UDPRequestHeader[3] = SOCKS5_ATYP_IPV6;
+			memcpy (m_UDPRequestHeader + 4, to.address ().to_v6().to_bytes().data(), 16);
+			requestHeaderSize = SOCKS5_UDP_IPV6_REQUEST_HEADER_SIZE;
+		}
+		else
+		{
+			m_UDPRequestHeader[3] = SOCKS5_ATYP_IPV4;
+			memcpy (m_UDPRequestHeader + 4, to.address ().to_v4().to_bytes().data(), 4);
+			requestHeaderSize = SOCKS5_UDP_IPV4_REQUEST_HEADER_SIZE;
+		}
+		htobe16buf (m_UDPRequestHeader + requestHeaderSize - 2, to.port ());
+
+		std::vector<boost::asio::const_buffer> bufs;
+		bufs.push_back (boost::asio::buffer (m_UDPRequestHeader, requestHeaderSize));
+		bufs.push_back (boost::asio::buffer (header, headerLen));
+		if (headerX) bufs.push_back (boost::asio::buffer (headerX, headerXLen));
+		bufs.push_back (boost::asio::buffer (payload, payloadLen));
+
+		boost::system::error_code ec;
+		m_SocketV4.send_to (bufs, *m_ProxyRelayEndpoint, 0, ec); // TODO: implement ipv6 proxy
+		if (!ec)
+			i2p::transport::transports.UpdateSentBytes (headerLen + payloadLen);
+		else
+			LogPrint (eLogError, "SSU2: Send exception: ", ec.message (), " to ", to);
+	}
+
+	void SSU2Server::ProcessNextPacketFromProxy (uint8_t * buf, size_t len)
+	{
+		if (buf[2]) // FRAG
+		{
+			LogPrint (eLogWarning, "SSU2: Proxy packet fragmentation is not supported");
+			return;
+		}
+		size_t offset = 0;
+		boost::asio::ip::udp::endpoint ep;
+		switch (buf[3]) // ATYP
+		{
+			case SOCKS5_ATYP_IPV4:
+			{
+				offset = SOCKS5_UDP_IPV4_REQUEST_HEADER_SIZE;
+				if (offset > len) return;
+				boost::asio::ip::address_v4::bytes_type bytes;
+				memcpy (bytes.data (), buf + 4, 4);
+				uint16_t port = bufbe16toh (buf + 8);
+				ep = boost::asio::ip::udp::endpoint (boost::asio::ip::address_v4 (bytes), port);
+				break;
+			}
+			case SOCKS5_ATYP_IPV6:
+			{
+				offset = SOCKS5_UDP_IPV6_REQUEST_HEADER_SIZE;
+				if (offset > len) return;
+				boost::asio::ip::address_v6::bytes_type bytes;
+				memcpy (bytes.data (), buf + 4, 16);
+				uint16_t port = bufbe16toh (buf + 20);
+				ep = boost::asio::ip::udp::endpoint (boost::asio::ip::address_v6 (bytes), port);
+				break;
+			}
+			default:
+			{
+				LogPrint (eLogWarning, "SSU2: Unknown ATYP ", (int)buf[3], " from proxy relay");
+				return;
+			}
+		}
+		ProcessNextPacket (buf + offset, len - offset, ep);
+	}
+
+	void SSU2Server::ConnectToProxy ()
+	{
+		if (!m_ProxyEndpoint) return;
+		m_UDPAssociateSocket.reset (new boost::asio::ip::tcp::socket (m_ReceiveService.GetService ()));
+		m_UDPAssociateSocket->async_connect (*m_ProxyEndpoint,
+		    [this] (const boost::system::error_code& ecode)
+			{
+				if (ecode)
+				{
+					LogPrint (eLogError, "SSU2: Can't connect to proxy ", *m_ProxyEndpoint, " ", ecode.message ());
+					m_UDPAssociateSocket.reset (nullptr);
+					ReconnectToProxy ();
+				}
+				else
+					HandshakeWithProxy ();
+			});
+	}
+
+	void SSU2Server::HandshakeWithProxy ()
+	{
+		if (!m_UDPAssociateSocket) return;
+		m_UDPRequestHeader[0] = SOCKS5_VER;
+		m_UDPRequestHeader[1] = 1; // 1 method
+		m_UDPRequestHeader[2] = 0; // no authentication
+		boost::asio::async_write (*m_UDPAssociateSocket, boost::asio::buffer (m_UDPRequestHeader, 3), boost::asio::transfer_all(),
+			[this] (const boost::system::error_code& ecode, std::size_t bytes_transferred)
+			{
+				(void) bytes_transferred;
+				if (ecode)
+				{
+					LogPrint(eLogError, "SSU2: Proxy write error ", ecode.message());
+					m_UDPAssociateSocket.reset (nullptr);
+					ReconnectToProxy ();
+				}
+				else
+					ReadHandshakeWithProxyReply ();
+			});
+	}
+
+	void SSU2Server::ReadHandshakeWithProxyReply ()
+	{
+		if (!m_UDPAssociateSocket) return;
+		boost::asio::async_read (*m_UDPAssociateSocket, boost::asio::buffer (m_UDPRequestHeader, 2), boost::asio::transfer_all(),
+			[this] (const boost::system::error_code& ecode, std::size_t bytes_transferred)
+			{
+				(void) bytes_transferred;
+				if (ecode)
+				{
+					LogPrint(eLogError, "SSU2: Proxy read error ", ecode.message());
+					m_UDPAssociateSocket.reset (nullptr);
+					ReconnectToProxy ();
+				}
+				else
+				{
+					if (m_UDPRequestHeader[0] == SOCKS5_VER && !m_UDPRequestHeader[1])
+						SendUDPAssociateRequest ();
+					else
+					{
+						LogPrint(eLogError, "SSU2: Invalid proxy reply");
+						m_UDPAssociateSocket.reset (nullptr);
+					}
+				}
+			});
+	}
+
+	void SSU2Server::SendUDPAssociateRequest ()
+	{
+		if (!m_UDPAssociateSocket) return;
+		m_UDPRequestHeader[0] = SOCKS5_VER;
+		m_UDPRequestHeader[1] = SOCKS5_CMD_UDP_ASSOCIATE;
+		m_UDPRequestHeader[2] = 0; // RSV
+		m_UDPRequestHeader[3] = SOCKS5_ATYP_IPV4; // TODO: implement ipv6 proxy
+		memset (m_UDPRequestHeader + 4, 0, 6); // address and port all zeros
+		boost::asio::async_write (*m_UDPAssociateSocket, boost::asio::buffer (m_UDPRequestHeader, SOCKS5_UDP_IPV4_REQUEST_HEADER_SIZE), boost::asio::transfer_all(),
+			[this] (const boost::system::error_code& ecode, std::size_t bytes_transferred)
+			{
+				(void) bytes_transferred;
+				if (ecode)
+				{
+					LogPrint(eLogError, "SSU2: Proxy write error ", ecode.message());
+					m_UDPAssociateSocket.reset (nullptr);
+					ReconnectToProxy ();
+				}
+				else
+					ReadUDPAssociateReply ();
+			});
+	}
+
+	void SSU2Server::ReadUDPAssociateReply ()
+	{
+		if (!m_UDPAssociateSocket) return;
+		boost::asio::async_read (*m_UDPAssociateSocket, boost::asio::buffer (m_UDPRequestHeader, SOCKS5_UDP_IPV4_REQUEST_HEADER_SIZE), boost::asio::transfer_all(),
+			[this] (const boost::system::error_code& ecode, std::size_t bytes_transferred)
+			{
+				(void) bytes_transferred;
+				if (ecode)
+				{
+					LogPrint(eLogError, "SSU2: Proxy read error ", ecode.message());
+					m_UDPAssociateSocket.reset (nullptr);
+					ReconnectToProxy ();
+				}
+				else
+				{
+					if (m_UDPRequestHeader[0] == SOCKS5_VER && !m_UDPRequestHeader[1])
+					{
+						if (m_UDPRequestHeader[3] == SOCKS5_ATYP_IPV4)
+						{
+							boost::asio::ip::address_v4::bytes_type bytes;
+							memcpy (bytes.data (), m_UDPRequestHeader + 4, 4);
+							uint16_t port = bufbe16toh (m_UDPRequestHeader + 8);
+							m_ProxyRelayEndpoint.reset (new boost::asio::ip::udp::endpoint (boost::asio::ip::address_v4 (bytes), port));
+							m_SocketV4.open (boost::asio::ip::udp::v4 ());
+							Receive (m_SocketV4);
+							ReadUDPAssociateSocket ();
+						}
+						else
+						{
+							LogPrint(eLogError, "SSU2: Proxy UDP associate unsupported ATYP ", (int)m_UDPRequestHeader[3]);
+							m_UDPAssociateSocket.reset (nullptr);
+						}
+					}
+					else
+					{
+						LogPrint(eLogError, "SSU2: Proxy UDP associate error ", (int)m_UDPRequestHeader[1]);
+						m_UDPAssociateSocket.reset (nullptr);
+					}
+				}
+			});
+	}
+
+	void SSU2Server::ReadUDPAssociateSocket ()
+	{
+		if (!m_UDPAssociateSocket) return;
+		m_UDPAssociateSocket->async_read_some (boost::asio::buffer (m_UDPRequestHeader, 1),
+			[this] (const boost::system::error_code& ecode, std::size_t bytes_transferred)
+			{
+				(void) bytes_transferred;
+				if (ecode)
+				{
+					LogPrint(eLogWarning, "SSU2: Proxy UDP Associate socket error ", ecode.message());
+					m_UDPAssociateSocket.reset (nullptr);
+					m_ProxyRelayEndpoint.reset (nullptr);
+					m_SocketV4.close ();
+					ConnectToProxy (); // try to reconnect immediately
+				}
+				else
+					ReadUDPAssociateSocket ();
+			});
+	}
+
+	void SSU2Server::ReconnectToProxy ()
+	{
+		LogPrint(eLogInfo, "SSU2: Reconnect to proxy after ", SSU2_PROXY_CONNECT_RETRY_TIMEOUT, " seconds");
+		if (m_ProxyConnectRetryTimer)
+			m_ProxyConnectRetryTimer->cancel ();
+		else
+			m_ProxyConnectRetryTimer.reset (new boost::asio::deadline_timer (m_ReceiveService.GetService ()));
+		m_ProxyConnectRetryTimer->expires_from_now (boost::posix_time::seconds (SSU2_PROXY_CONNECT_RETRY_TIMEOUT));
+		m_ProxyConnectRetryTimer->async_wait (
+			[this](const boost::system::error_code& ecode)
+			{
+				if (ecode != boost::asio::error::operation_aborted)
+				{
+					m_UDPAssociateSocket.reset (nullptr);
+					m_ProxyRelayEndpoint.reset (nullptr);
+					LogPrint(eLogInfo, "SSU2: Reconnecting to proxy");
+					ConnectToProxy ();
+				}
+			});
+	}
+
+	bool SSU2Server::SetProxy (const std::string& address, uint16_t port)
+	{
+		boost::system::error_code ecode;
+		auto addr = boost::asio::ip::address::from_string (address, ecode);
+		if (!ecode && !addr.is_unspecified () && port)
+		{
+			m_IsThroughProxy = true;
+			m_ProxyEndpoint.reset (new boost::asio::ip::tcp::endpoint (addr, port));
+		}
+		else
+		{
+			if (ecode)
+				LogPrint (eLogError, "SSU2: Invalid proxy address ", address, " ", ecode.message());
+			return false;
+		}
+		return true;
+	}
 }
 }
