@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2022, The PurpleI2P Project
+* Copyright (c) 2013-2023, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -136,12 +136,14 @@ namespace transport
 	Transports::Transports ():
 		m_IsOnline (true), m_IsRunning (false), m_IsNAT (true), m_CheckReserved(true), m_Thread (nullptr),
 		m_Service (nullptr), m_Work (nullptr), m_PeerCleanupTimer (nullptr), m_PeerTestTimer (nullptr),
-		m_SSUServer (nullptr), m_SSU2Server (nullptr), m_NTCP2Server (nullptr),
+		m_UpdateBandwidthTimer (nullptr), m_SSU2Server (nullptr), m_NTCP2Server (nullptr),
 		m_X25519KeysPairSupplier (15), // 15 pre-generated keys
-		m_TotalSentBytes(0), m_TotalReceivedBytes(0), m_TotalTransitTransmittedBytes (0),
-		m_InBandwidth (0), m_OutBandwidth (0), m_TransitBandwidth(0),
-		m_LastInBandwidthUpdateBytes (0), m_LastOutBandwidthUpdateBytes (0),
-		m_LastTransitBandwidthUpdateBytes (0), m_LastBandwidthUpdateTime (0)
+		m_TotalSentBytes (0), m_TotalReceivedBytes (0), m_TotalTransitTransmittedBytes (0),
+		m_InBandwidth (0), m_OutBandwidth (0), m_TransitBandwidth (0),
+		m_LastInBandwidthUpdateBytes (0), m_LastOutBandwidthUpdateBytes (0), m_LastTransitBandwidthUpdateBytes (0),
+		m_InBandwidth15s (0), m_OutBandwidth15s (0), m_TransitBandwidth15s (0),
+		m_LastInBandwidth15sUpdateBytes (0), m_LastOutBandwidth15sUpdateBytes (0), m_LastTransitBandwidth15sUpdateBytes (0),
+		m_LastBandwidth15sUpdateTime (0)
 	{
 	}
 
@@ -152,12 +154,13 @@ namespace transport
 		{
 			delete m_PeerCleanupTimer; m_PeerCleanupTimer = nullptr;
 			delete m_PeerTestTimer; m_PeerTestTimer = nullptr;
+			delete m_UpdateBandwidthTimer; m_UpdateBandwidthTimer = nullptr;
 			delete m_Work; m_Work = nullptr;
 			delete m_Service; m_Service = nullptr;
 		}
 	}
 
-	void Transports::Start (bool enableNTCP2, bool enableSSU, bool enableSSU2)
+	void Transports::Start (bool enableNTCP2, bool enableSSU2)
 	{
 		if (!m_Service)
 		{
@@ -165,8 +168,11 @@ namespace transport
 			m_Work = new boost::asio::io_service::work (*m_Service);
 			m_PeerCleanupTimer = new boost::asio::deadline_timer (*m_Service);
 			m_PeerTestTimer = new boost::asio::deadline_timer (*m_Service);
+			m_UpdateBandwidthTimer = new boost::asio::deadline_timer (*m_Service);
 		}
 
+		bool ipv4; i2p::config::GetOption("ipv4", ipv4);
+		bool ipv6; i2p::config::GetOption("ipv6", ipv6);
 		i2p::config::GetOption("nat", m_IsNAT);
 		m_X25519KeysPairSupplier.Start ();
 		m_IsRunning = true;
@@ -190,6 +196,8 @@ namespace transport
 
 						m_NTCP2Server->UseProxy(proxytype, proxyurl.host, proxyurl.port, proxyurl.user, proxyurl.pass);
 						i2p::context.SetStatus (eRouterStatusProxy);
+						if (ipv6)
+							i2p::context.SetStatusV6 (eRouterStatusProxy);
 					}
 					else
 						LogPrint(eLogError, "Transports: Unsupported NTCP2 proxy URL ", ntcp2proxy);
@@ -201,27 +209,30 @@ namespace transport
 				m_NTCP2Server = new NTCP2Server ();
 		}
 
-		// create SSU server
-		int ssuPort = 0;
-		if (enableSSU)
+		// create SSU2 server
+		if (enableSSU2)
 		{
-			auto& addresses = context.GetRouterInfo ().GetAddresses ();
-			for (const auto& address: addresses)
+			m_SSU2Server = new SSU2Server ();
+			std::string ssu2proxy; i2p::config::GetOption("ssu2.proxy", ssu2proxy);
+			if (!ssu2proxy.empty())
 			{
-				if (!address) continue;
-				if (address->transportStyle == RouterInfo::eTransportSSU)
+				if (proxyurl.parse (ssu2proxy) && proxyurl.schema == "socks")
 				{
-					ssuPort = address->port;
-					m_SSUServer = new SSUServer (address->port);
-					break;
+					if (m_SSU2Server->SetProxy (proxyurl.host, proxyurl.port))
+					{
+						i2p::context.SetStatus (eRouterStatusProxy);
+						if (ipv6)
+							i2p::context.SetStatusV6 (eRouterStatusProxy);
+					}
+					else
+						LogPrint(eLogError, "Transports: Can't set SSU2 proxy ", ssu2proxy);
 				}
+				else
+					LogPrint(eLogError, "Transports: Invalid SSU2 proxy URL ", ssu2proxy);
 			}
 		}
-		// create SSU2 server
-		if (enableSSU2) m_SSU2Server = new SSU2Server ();
 
 		// bind to interfaces
-		bool ipv4; i2p::config::GetOption("ipv4", ipv4);
 		if (ipv4)
 		{
 			std::string address; i2p::config::GetOption("address4", address);
@@ -232,13 +243,22 @@ namespace transport
 				if (!ec)
 				{
 					if (m_NTCP2Server) m_NTCP2Server->SetLocalAddress (addr);
-					if (m_SSUServer) m_SSUServer->SetLocalAddress (addr);
 					if (m_SSU2Server) m_SSU2Server->SetLocalAddress (addr);
 				}
 			}
+
+			if (enableSSU2)
+			{
+				uint16_t mtu; i2p::config::GetOption ("ssu2.mtu4", mtu);
+				if (mtu)
+				{
+					if (mtu < (int)SSU2_MIN_PACKET_SIZE) mtu = SSU2_MIN_PACKET_SIZE;
+					if (mtu > (int)SSU2_MAX_PACKET_SIZE) mtu = SSU2_MAX_PACKET_SIZE;
+					i2p::context.SetMTU (mtu, true);
+				}
+			}
 		}
 
-		bool ipv6; i2p::config::GetOption("ipv6", ipv6);
 		if (ipv6)
 		{
 			std::string address; i2p::config::GetOption("address6", address);
@@ -249,10 +269,20 @@ namespace transport
 				if (!ec)
 				{
 					if (m_NTCP2Server) m_NTCP2Server->SetLocalAddress (addr);
-					if (m_SSUServer) m_SSUServer->SetLocalAddress (addr);
 					if (m_SSU2Server) m_SSU2Server->SetLocalAddress (addr);
 				}
 			}
+
+			if (enableSSU2)
+			{
+				uint16_t mtu; i2p::config::GetOption ("ssu2.mtu6", mtu);
+				if (mtu)
+				{
+					if (mtu < (int)SSU2_MIN_PACKET_SIZE) mtu = SSU2_MIN_PACKET_SIZE;
+					if (mtu > (int)SSU2_MAX_PACKET_SIZE) mtu = SSU2_MAX_PACKET_SIZE;
+					i2p::context.SetMTU (mtu, false);
+				}
+			}
 		}
 
 		bool ygg; i2p::config::GetOption("meshnets.yggdrasil", ygg);
@@ -271,26 +301,14 @@ namespace transport
 		// start servers
 		if (m_NTCP2Server) m_NTCP2Server->Start ();
 		if (m_SSU2Server) m_SSU2Server->Start ();
-		if (m_SSUServer)
-		{
-			LogPrint (eLogInfo, "Transports: Start listening UDP port ", ssuPort);
-			try
-			{
-				m_SSUServer->Start ();
-			}
-			catch (std::exception& ex )
-			{
-				LogPrint(eLogError, "Transports: Failed to bind to UDP port", ssuPort);
-				m_SSUServer->Stop ();
-				delete m_SSUServer;
-				m_SSUServer = nullptr;
-			}
-		}
-		if (m_SSUServer || m_SSU2Server) DetectExternalIP ();
+		if (m_SSU2Server) DetectExternalIP ();
 
-		m_PeerCleanupTimer->expires_from_now (boost::posix_time::seconds(5*SESSION_CREATION_TIMEOUT));
+		m_PeerCleanupTimer->expires_from_now (boost::posix_time::seconds(5 * SESSION_CREATION_TIMEOUT));
 		m_PeerCleanupTimer->async_wait (std::bind (&Transports::HandlePeerCleanupTimer, this, std::placeholders::_1));
 
+		m_UpdateBandwidthTimer->expires_from_now (boost::posix_time::seconds(1));
+		m_UpdateBandwidthTimer->async_wait (std::bind (&Transports::HandleUpdateBandwidthTimer, this, std::placeholders::_1));
+
 		if (m_IsNAT)
 		{
 			m_PeerTestTimer->expires_from_now (boost::posix_time::minutes(PEER_TEST_INTERVAL));
@@ -303,12 +321,6 @@ namespace transport
 		if (m_PeerCleanupTimer) m_PeerCleanupTimer->cancel ();
 		if (m_PeerTestTimer) m_PeerTestTimer->cancel ();
 		m_Peers.clear ();
-		if (m_SSUServer)
-		{
-			m_SSUServer->Stop ();
-			delete m_SSUServer;
-			m_SSUServer = nullptr;
-		}
 
 		if (m_SSU2Server)
 		{
@@ -352,29 +364,44 @@ namespace transport
 		}
 	}
 
-	void Transports::UpdateBandwidth ()
+	void Transports::HandleUpdateBandwidthTimer (const boost::system::error_code& ecode)
 	{
-		uint64_t ts = i2p::util::GetMillisecondsSinceEpoch ();
-		if (m_LastBandwidthUpdateTime > 0)
+		if (ecode != boost::asio::error::operation_aborted)
 		{
-			auto delta = ts - m_LastBandwidthUpdateTime;
-			if (delta > 0)
+			uint64_t ts = i2p::util::GetMillisecondsSinceEpoch ();
+
+			// updated every second
+			m_InBandwidth = m_TotalReceivedBytes - m_LastInBandwidthUpdateBytes;
+			m_OutBandwidth = m_TotalSentBytes - m_LastOutBandwidthUpdateBytes;
+			m_TransitBandwidth = m_TotalTransitTransmittedBytes - m_LastTransitBandwidthUpdateBytes;
+
+			m_LastInBandwidthUpdateBytes = m_TotalReceivedBytes;
+			m_LastOutBandwidthUpdateBytes = m_TotalSentBytes;
+			m_LastTransitBandwidthUpdateBytes = m_TotalTransitTransmittedBytes;
+
+			// updated every 15 seconds
+			auto delta = ts - m_LastBandwidth15sUpdateTime;
+			if (delta > 15 * 1000)
 			{
-				m_InBandwidth = (m_TotalReceivedBytes - m_LastInBandwidthUpdateBytes)*1000/delta; // per second
-				m_OutBandwidth = (m_TotalSentBytes - m_LastOutBandwidthUpdateBytes)*1000/delta; // per second
-				m_TransitBandwidth = (m_TotalTransitTransmittedBytes - m_LastTransitBandwidthUpdateBytes)*1000/delta;
+				m_InBandwidth15s = (m_TotalReceivedBytes - m_LastInBandwidth15sUpdateBytes) * 1000 / delta;
+				m_OutBandwidth15s = (m_TotalSentBytes - m_LastOutBandwidth15sUpdateBytes) * 1000 / delta;
+				m_TransitBandwidth15s = (m_TotalTransitTransmittedBytes - m_LastTransitBandwidth15sUpdateBytes) * 1000 / delta;
+
+				m_LastBandwidth15sUpdateTime = ts;
+				m_LastInBandwidth15sUpdateBytes = m_TotalReceivedBytes;
+				m_LastOutBandwidth15sUpdateBytes = m_TotalSentBytes;
+				m_LastTransitBandwidth15sUpdateBytes = m_TotalTransitTransmittedBytes;
 			}
+
+			m_UpdateBandwidthTimer->expires_from_now (boost::posix_time::seconds(1));
+			m_UpdateBandwidthTimer->async_wait (std::bind (&Transports::HandleUpdateBandwidthTimer, this, std::placeholders::_1));
 		}
-		m_LastBandwidthUpdateTime = ts;
-		m_LastInBandwidthUpdateBytes = m_TotalReceivedBytes;
-		m_LastOutBandwidthUpdateBytes = m_TotalSentBytes;
-		m_LastTransitBandwidthUpdateBytes = m_TotalTransitTransmittedBytes;
 	}
 
 	bool Transports::IsBandwidthExceeded () const
 	{
 		auto limit = i2p::context.GetBandwidthLimit() * 1024; // convert to bytes
-		auto bw = std::max (m_InBandwidth, m_OutBandwidth);
+		auto bw = std::max (m_InBandwidth15s, m_OutBandwidth15s);
 		return bw > limit;
 	}
 
@@ -417,8 +444,7 @@ namespace transport
 				{
 					auto ts = i2p::util::GetSecondsSinceEpoch ();
 					std::unique_lock<std::mutex> l(m_PeersMutex);
-					it = m_Peers.insert (std::pair<i2p::data::IdentHash, Peer>(ident, { 0, r, {},
-						ts, ts + PEER_ROUTER_INFO_UPDATE_INTERVAL, {} })).first;
+					it = m_Peers.insert (std::pair<i2p::data::IdentHash, Peer>(ident, {r, ts})).first;
 				}
 				connected = ConnectToPeer (ident, it->second);
 			}
@@ -453,127 +479,66 @@ namespace transport
 			peer.router = netdb.FindRouter (ident); // try to get new one from netdb
 		if (peer.router) // we have RI already
 		{
-			if (peer.numAttempts < 2) // NTCP2, 0 - ipv6, 1 - ipv4
-			{
-				if (m_NTCP2Server) // we support NTCP2
-				{
-					std::shared_ptr<const RouterInfo::Address> address;
-					if (!peer.numAttempts) // NTCP2 ipv6
-					{
-						if (context.GetRouterInfo ().IsNTCP2V6 () && peer.router->IsReachableBy (RouterInfo::eNTCP2V6))
-						{
-							address = peer.router->GetPublishedNTCP2V6Address ();
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (!address && peer.numAttempts == 1) // NTCP2 ipv4
-					{
-						if (context.GetRouterInfo ().IsNTCP2 (true) && peer.router->IsReachableBy (RouterInfo::eNTCP2V4))
-						{
-							address = peer.router->GetPublishedNTCP2V4Address ();
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (address)
-					{
-						auto s = std::make_shared<NTCP2Session> (*m_NTCP2Server, peer.router, address);
-						if( m_NTCP2Server->UsingProxy())
-							m_NTCP2Server->ConnectWithProxy(s);
-						else
-							m_NTCP2Server->Connect (s);
-						return true;
-					}
-				}
-				else
-					peer.numAttempts = 2; // switch to SSU
-			}
-			if (peer.numAttempts == 2 || peer.numAttempts == 3) // SSU2, 2 - ipv6, 3 - ipv4
-			{
-				if (m_SSU2Server)
-				{
-					std::shared_ptr<const RouterInfo::Address> address;
-					if (peer.numAttempts == 2) // SSU2 ipv6
-					{
-						if (context.GetRouterInfo ().IsSSU2V6 () && peer.router->IsReachableBy (RouterInfo::eSSU2V6))
-						{
-							address = peer.router->GetSSU2V6Address ();
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (!address && peer.numAttempts == 3) // SSU2 ipv4
-					{
-						if (context.GetRouterInfo ().IsSSU2V4 () && peer.router->IsReachableBy (RouterInfo::eSSU2V4))
-						{
-							address = peer.router->GetSSU2V4Address ();
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (address && address->IsReachableSSU ())
-					{
-						if (m_SSU2Server->CreateSession (peer.router, address))
-							return true;
-					}
-				}
-				else
-					peer.numAttempts += 2; // switch to mesh
-			}
-			if (peer.numAttempts == 4) // Mesh
+			if (peer.priority.empty ())
+				SetPriority (peer);
+			while (peer.numAttempts < (int)peer.priority.size ())
 			{
+				auto tr = peer.priority[peer.numAttempts];
 				peer.numAttempts++;
-				if (m_NTCP2Server && context.GetRouterInfo ().IsMesh () && peer.router->IsMesh ())
+				switch (tr)
 				{
-					auto address = peer.router->GetYggdrasilAddress ();
-					if (address)
+					case i2p::data::RouterInfo::eNTCP2V4:
+					case i2p::data::RouterInfo::eNTCP2V6:
 					{
-						auto s = std::make_shared<NTCP2Session> (*m_NTCP2Server, peer.router, address);
-						m_NTCP2Server->Connect (s);
-						return true;
-					}
-				}
-			}
-			if (peer.numAttempts == 5 || peer.numAttempts == 6) // SSU, 5 - ipv6, 6 - ipv4
-			{
-				if (m_SSUServer)
-				{
-					std::shared_ptr<const RouterInfo::Address> address;
-					if (peer.numAttempts == 5) // SSU ipv6
-					{
-						if (context.GetRouterInfo ().IsSSUV6 () && peer.router->IsReachableBy (RouterInfo::eSSUV6))
+						if (!m_NTCP2Server) continue;
+						std::shared_ptr<const RouterInfo::Address> address = (tr == i2p::data::RouterInfo::eNTCP2V6) ?
+							peer.router->GetPublishedNTCP2V6Address () : peer.router->GetPublishedNTCP2V4Address ();
+						if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
+							address = nullptr;
+						if (address)
 						{
-							address = peer.router->GetSSUV6Address ();
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (!address && peer.numAttempts == 6) // SSU ipv4
-					{
-						if (context.GetRouterInfo ().IsSSU (true) && peer.router->IsReachableBy (RouterInfo::eSSUV4))
-						{
-							address = peer.router->GetSSUAddress (true);
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (address && address->IsReachableSSU ())
-					{
-						if (m_SSUServer->CreateSession (peer.router, address))
+							auto s = std::make_shared<NTCP2Session> (*m_NTCP2Server, peer.router, address);
+							if( m_NTCP2Server->UsingProxy())
+								m_NTCP2Server->ConnectWithProxy(s);
+							else
+								m_NTCP2Server->Connect (s);
 							return true;
+						}
+						break;
 					}
+					case i2p::data::RouterInfo::eSSU2V4:
+					case i2p::data::RouterInfo::eSSU2V6:
+					{
+						if (!m_SSU2Server) continue;
+						std::shared_ptr<const RouterInfo::Address> address = (tr == i2p::data::RouterInfo::eSSU2V6) ?
+							peer.router->GetSSU2V6Address () : peer.router->GetSSU2V4Address ();
+						if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
+							address = nullptr;
+						if (address && address->IsReachableSSU ())
+						{
+							if (m_SSU2Server->CreateSession (peer.router, address))
+								return true;
+						}
+						break;
+					}
+					case i2p::data::RouterInfo::eNTCP2V6Mesh:
+					{
+						if (!m_NTCP2Server) continue;
+						auto address = peer.router->GetYggdrasilAddress ();
+						if (address)
+						{
+							auto s = std::make_shared<NTCP2Session> (*m_NTCP2Server, peer.router, address);
+							m_NTCP2Server->Connect (s);
+							return true;
+						}
+						break;
+					}
+					default:
+						LogPrint (eLogError, "Transports: Unknown transport ", (int)tr);
 				}
-				else
-					peer.numAttempts += 2; 
 			}
-			LogPrint (eLogInfo, "Transports: No compatble NTCP2 or SSU addresses available");
+
+			LogPrint (eLogInfo, "Transports: No compatible addresses available");
 			i2p::data::netdb.SetUnreachable (ident, true); // we are here because all connection attempts failed
 			peer.Done ();
 			std::unique_lock<std::mutex> l(m_PeersMutex);
@@ -589,6 +554,37 @@ namespace transport
 		return true;
 	}
 
+	void Transports::SetPriority (Peer& peer) const
+	{
+		static const std::vector<i2p::data::RouterInfo::SupportedTransports>
+			ntcp2Priority =
+		{
+			i2p::data::RouterInfo::eNTCP2V6,
+			i2p::data::RouterInfo::eNTCP2V4,
+			i2p::data::RouterInfo::eSSU2V6,
+			i2p::data::RouterInfo::eSSU2V4,
+			i2p::data::RouterInfo::eNTCP2V6Mesh
+		},
+			ssu2Priority =
+		{
+			i2p::data::RouterInfo::eSSU2V6,
+			i2p::data::RouterInfo::eSSU2V4,
+			i2p::data::RouterInfo::eNTCP2V6,
+			i2p::data::RouterInfo::eNTCP2V4,
+			i2p::data::RouterInfo::eNTCP2V6Mesh
+		};
+		if (!peer.router) return;
+		auto compatibleTransports = context.GetRouterInfo ().GetCompatibleTransports (false) &
+			peer.router->GetCompatibleTransports (true);
+		peer.numAttempts = 0;
+		peer.priority.clear ();
+		bool ssu2 = rand () & 1;
+		const auto& priority = ssu2 ? ssu2Priority : ntcp2Priority;
+		for (auto transport: priority)
+			if (transport & compatibleTransports)
+				peer.priority.push_back (transport);
+	}
+
 	void Transports::RequestComplete (std::shared_ptr<const i2p::data::RouterInfo> r, const i2p::data::IdentHash& ident)
 	{
 		m_Service->post (std::bind (&Transports::HandleRequestComplete, this, r, ident));
@@ -622,7 +618,7 @@ namespace transport
 			i2p::context.SetStatus (eRouterStatusOK);
 			return;
 		}
-		if (m_SSUServer || m_SSU2Server)
+		if (m_SSU2Server)
 			PeerTest ();
 		else
 			LogPrint (eLogWarning, "Transports: Can't detect external IP. SSU or SSU2 is not available");
@@ -630,101 +626,44 @@ namespace transport
 
 	void Transports::PeerTest (bool ipv4, bool ipv6)
 	{
-		if (RoutesRestricted() || (!m_SSUServer && !m_SSU2Server)) return;
+		if (RoutesRestricted() || !m_SSU2Server || m_SSU2Server->UsesProxy ()) return;
 		if (ipv4 && i2p::context.SupportsV4 ())
 		{
 			LogPrint (eLogInfo, "Transports: Started peer test IPv4");
 			std::set<i2p::data::IdentHash> excluded;
 			excluded.insert (i2p::context.GetIdentHash ()); // don't pick own router
-			if (m_SSUServer)
+			for (int i = 0; i < 5; i++)
 			{
-				bool statusChanged = false;
-				for (int i = 0; i < 5; i++)
+				auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (true, excluded); // v4
+				if (router)
 				{
-					auto router = i2p::data::netdb.GetRandomPeerTestRouter (true, excluded); // v4
-					if (router)
-					{
-						auto addr = router->GetSSUAddress (true); // ipv4
-						if (addr && !i2p::util::net::IsInReservedRange(addr->host))
-						{
-							if (!statusChanged)
-							{
-								statusChanged = true;
-								i2p::context.SetStatus (eRouterStatusTesting); // first time only
-							}
-							m_SSUServer->CreateSession (router, addr, true); // peer test v4
-						}
-						excluded.insert (router->GetIdentHash ());
-					}
-				}
-				if (!statusChanged)
-					LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv4");
-			}
-			// SSU2
-			if (m_SSU2Server)
-			{
-				excluded.clear ();
-				excluded.insert (i2p::context.GetIdentHash ());
-				for (int i = 0; i < 3; i++)
-				{
-					auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (true, excluded); // v4
-					if (router)
-					{
-						if (i2p::context.GetStatus () != eRouterStatusTesting)
-							i2p::context.SetStatusSSU2 (eRouterStatusTesting);
-						m_SSU2Server->StartPeerTest (router, true);
-						excluded.insert (router->GetIdentHash ());
-					}
+					if (i2p::context.GetStatus () != eRouterStatusTesting)
+						i2p::context.SetStatus (eRouterStatusTesting);
+					m_SSU2Server->StartPeerTest (router, true);
+					excluded.insert (router->GetIdentHash ());
 				}
 			}
+			if (excluded.size () <= 1)
+				LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv4");
 		}
 		if (ipv6 && i2p::context.SupportsV6 ())
 		{
 			LogPrint (eLogInfo, "Transports: Started peer test IPv6");
 			std::set<i2p::data::IdentHash> excluded;
 			excluded.insert (i2p::context.GetIdentHash ()); // don't pick own router
-			if (m_SSUServer)
+			for (int i = 0; i < 5; i++)
 			{
-				bool statusChanged = false;
-				for (int i = 0; i < 5; i++)
+				auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (false, excluded); // v6
+				if (router)
 				{
-					auto router = i2p::data::netdb.GetRandomPeerTestRouter (false, excluded); // v6
-					if (router)
-					{
-						auto addr = router->GetSSUV6Address ();
-						if (addr && !i2p::util::net::IsInReservedRange(addr->host))
-						{
-							if (!statusChanged)
-							{
-								statusChanged = true;
-								i2p::context.SetStatusV6 (eRouterStatusTesting); // first time only
-							}
-							m_SSUServer->CreateSession (router, addr, true); // peer test v6
-						}
-						excluded.insert (router->GetIdentHash ());
-					}
-				}
-				if (!statusChanged)
-					LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv6");
-			}
-
-			// SSU2
-			if (m_SSU2Server)
-			{
-				excluded.clear ();
-				excluded.insert (i2p::context.GetIdentHash ());
-				for (int i = 0; i < 3; i++)
-				{
-					auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (false, excluded); // v6
-					if (router)
-					{
-						if (i2p::context.GetStatusV6 () != eRouterStatusTesting)
-							i2p::context.SetStatusV6SSU2 (eRouterStatusTesting);
-						m_SSU2Server->StartPeerTest (router, false);
-						excluded.insert (router->GetIdentHash ());
-					}
+					if (i2p::context.GetStatusV6 () != eRouterStatusTesting)
+						i2p::context.SetStatusV6 (eRouterStatusTesting);
+					m_SSU2Server->StartPeerTest (router, false);
+					excluded.insert (router->GetIdentHash ());
 				}
 			}
+			if (excluded.size () <= 1)
+				LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv6");
 		}
 	}
 
@@ -777,8 +716,8 @@ namespace transport
 				session->SendI2NPMessages ({ CreateDatabaseStoreMsg () }); // send DatabaseStore
 				auto ts = i2p::util::GetSecondsSinceEpoch ();
 				std::unique_lock<std::mutex> l(m_PeersMutex);
-				m_Peers.insert (std::make_pair (ident, Peer{ 0, nullptr, { session },
-					ts, ts + PEER_ROUTER_INFO_UPDATE_INTERVAL, {} }));
+				auto it = m_Peers.insert (std::make_pair (ident, Peer{ nullptr, ts })).first;
+				it->second.sessions.push_back (session);
 			}
 		});
 	}
@@ -827,7 +766,12 @@ namespace transport
 			auto ts = i2p::util::GetSecondsSinceEpoch ();
 			for (auto it = m_Peers.begin (); it != m_Peers.end (); )
 			{
-				if (it->second.sessions.empty () && ts > it->second.creationTime + SESSION_CREATION_TIMEOUT)
+				it->second.sessions.remove_if (
+					[](std::shared_ptr<TransportSession> session)->bool
+					{
+						return !session || !session->IsEstablished ();
+					});
+ 				if (it->second.sessions.empty () && ts > it->second.creationTime + SESSION_CREATION_TIMEOUT)
 				{
 					LogPrint (eLogWarning, "Transports: Session to peer ", it->first.ToBase64 (), " has not been created in ", SESSION_CREATION_TIMEOUT, " seconds");
 					auto profile = i2p::data::GetRouterProfile(it->first);
@@ -845,19 +789,18 @@ namespace transport
 						auto session = it->second.sessions.front ();
 						if (session)
 							session->SendLocalRouterInfo (true);
-						it->second.nextRouterInfoUpdateTime = ts + PEER_ROUTER_INFO_UPDATE_INTERVAL + 
+						it->second.nextRouterInfoUpdateTime = ts + PEER_ROUTER_INFO_UPDATE_INTERVAL +
 							rand () % PEER_ROUTER_INFO_UPDATE_INTERVAL_VARIANCE;
 					}
 					++it;
 				}
 			}
-			UpdateBandwidth (); // TODO: use separate timer(s) for it
 			bool ipv4Testing = i2p::context.GetStatus () == eRouterStatusTesting;
 			bool ipv6Testing = i2p::context.GetStatusV6 () == eRouterStatusTesting;
 			// if still testing, repeat peer test
 			if (ipv4Testing || ipv6Testing)
 				PeerTest (ipv4Testing, ipv6Testing);
-			m_PeerCleanupTimer->expires_from_now (boost::posix_time::seconds(3*SESSION_CREATION_TIMEOUT));
+			m_PeerCleanupTimer->expires_from_now (boost::posix_time::seconds(3 * SESSION_CREATION_TIMEOUT));
 			m_PeerCleanupTimer->async_wait (std::bind (&Transports::HandlePeerCleanupTimer, this, std::placeholders::_1));
 		}
 	}
@@ -880,7 +823,9 @@ namespace transport
 			std::unique_lock<std::mutex> l(m_PeersMutex);
 			auto it = m_Peers.begin ();
 			std::advance (it, rand () % m_Peers.size ());
-			if (it == m_Peers.end () || it->second.router) return nullptr; // not connected
+			if (it == m_Peers.end () || it->second.router || it->second.sessions.empty () ||
+			    it->second.sessions.front ()->GetSendQueueSize () > PEER_ROUTER_INFO_OVERLOAD_QUEUE_SIZE)
+				return nullptr; // not connected or overloaded
 			ident = it->first;
 		}
 		return i2p::data::netdb.FindRouter (ident);
@@ -975,5 +920,119 @@ namespace transport
 				i2p::context.SetError (eRouterErrorOffline);
 		}
 	}
+
+	void InitAddressFromIface ()
+	{
+		bool ipv6; i2p::config::GetOption("ipv6", ipv6);
+		bool ipv4; i2p::config::GetOption("ipv4", ipv4);
+
+		// ifname -> address
+		std::string ifname; i2p::config::GetOption("ifname", ifname);
+		if (ipv4 && i2p::config::IsDefault ("address4"))
+		{
+			std::string ifname4; i2p::config::GetOption("ifname4", ifname4);
+			if (!ifname4.empty ())
+				i2p::config::SetOption ("address4", i2p::util::net::GetInterfaceAddress(ifname4, false).to_string ()); // v4
+			else if (!ifname.empty ())
+				i2p::config::SetOption ("address4", i2p::util::net::GetInterfaceAddress(ifname, false).to_string ()); // v4
+		}
+		if (ipv6 && i2p::config::IsDefault ("address6"))
+		{
+			std::string ifname6; i2p::config::GetOption("ifname6", ifname6);
+			if (!ifname6.empty ())
+				i2p::config::SetOption ("address6", i2p::util::net::GetInterfaceAddress(ifname6, true).to_string ()); // v6
+			else if (!ifname.empty ())
+				i2p::config::SetOption ("address6", i2p::util::net::GetInterfaceAddress(ifname, true).to_string ()); // v6
+		}
+	}
+
+	void InitTransports ()
+	{
+		bool ipv6;     i2p::config::GetOption("ipv6", ipv6);
+		bool ipv4;     i2p::config::GetOption("ipv4", ipv4);
+		bool ygg;      i2p::config::GetOption("meshnets.yggdrasil", ygg);
+		uint16_t port; i2p::config::GetOption("port", port);
+
+		boost::asio::ip::address_v6 yggaddr;
+		if (ygg)
+		{
+			std::string yggaddress; i2p::config::GetOption ("meshnets.yggaddress", yggaddress);
+			if (!yggaddress.empty ())
+			{
+				yggaddr = boost::asio::ip::address_v6::from_string (yggaddress);
+				if (yggaddr.is_unspecified () || !i2p::util::net::IsYggdrasilAddress (yggaddr) ||
+					!i2p::util::net::IsLocalAddress (yggaddr))
+				{
+					LogPrint(eLogWarning, "Transports: Can't find Yggdrasil address ", yggaddress);
+					ygg = false;
+				}
+			}
+			else
+			{
+				yggaddr = i2p::util::net::GetYggdrasilAddress ();
+				if (yggaddr.is_unspecified ())
+				{
+					LogPrint(eLogWarning, "Transports: Yggdrasil is not running. Disabled");
+					ygg = false;
+				}
+			}
+		}
+
+		if (!i2p::config::IsDefault("port"))
+		{
+			LogPrint(eLogInfo, "Transports: Accepting incoming connections at port ", port);
+			i2p::context.UpdatePort (port);
+		}
+		i2p::context.SetSupportsV6 (ipv6);
+		i2p::context.SetSupportsV4 (ipv4);
+		i2p::context.SetSupportsMesh (ygg, yggaddr);
+
+		bool ntcp2; i2p::config::GetOption("ntcp2.enabled", ntcp2);
+		if (ntcp2)
+		{
+			bool published; i2p::config::GetOption("ntcp2.published", published);
+			if (published)
+			{
+				std::string ntcp2proxy; i2p::config::GetOption("ntcp2.proxy", ntcp2proxy);
+				if (!ntcp2proxy.empty ()) published = false;
+			}
+			if (published)
+			{
+				uint16_t ntcp2port; i2p::config::GetOption("ntcp2.port", ntcp2port);
+				if (!ntcp2port) ntcp2port = port; // use standard port
+				i2p::context.PublishNTCP2Address (ntcp2port, true, ipv4, ipv6, false); // publish
+				if (ipv6)
+				{
+					std::string ipv6Addr; i2p::config::GetOption("ntcp2.addressv6", ipv6Addr);
+					auto addr = boost::asio::ip::address_v6::from_string (ipv6Addr);
+					if (!addr.is_unspecified () && addr != boost::asio::ip::address_v6::any ())
+						i2p::context.UpdateNTCP2V6Address (addr); // set ipv6 address if configured
+				}
+			}
+			else
+				i2p::context.PublishNTCP2Address (port, false, ipv4, ipv6, false); // unpublish
+		}
+		if (ygg)
+		{
+			i2p::context.PublishNTCP2Address (port, true, false, false, true);
+			i2p::context.UpdateNTCP2V6Address (yggaddr);
+			if (!ipv4 && !ipv6)
+				i2p::context.SetStatus (eRouterStatusMesh);
+		}
+		bool ssu2; i2p::config::GetOption("ssu2.enabled", ssu2);
+		if (ssu2 && i2p::config::IsDefault ("ssu2.enabled") && !ipv4 && !ipv6)
+			ssu2 = false; // don't enable ssu2 for yggdrasil only router
+		if (ssu2)
+		{
+			uint16_t ssu2port; i2p::config::GetOption("ssu2.port", ssu2port);
+			if (!ssu2port && port) ssu2port = port;
+			bool published; i2p::config::GetOption("ssu2.published", published);
+			if (published)
+				i2p::context.PublishSSU2Address (ssu2port, true, ipv4, ipv6); // publish
+			else
+				i2p::context.PublishSSU2Address (ssu2port, false, ipv4, ipv6); // unpublish
+		}
+
+	}
 }
 }
