@@ -30,7 +30,7 @@ namespace tunnel
 		{
 			peers.push_back (r->GetRouterIdentity ());
 			if (r->GetVersion () < i2p::data::NETDB_MIN_SHORT_TUNNEL_BUILD_VERSION ||
-			    r->GetRouterIdentity ()->GetCryptoKeyType () != i2p::data::CRYPTO_KEY_TYPE_ECIES_X25519_AEAD)
+				r->GetRouterIdentity ()->GetCryptoKeyType () != i2p::data::CRYPTO_KEY_TYPE_ECIES_X25519_AEAD)
 				isShort = false;
 		}
 	}
@@ -40,15 +40,25 @@ namespace tunnel
 		std::reverse (peers.begin (), peers.end ());
 	}
 
-	TunnelPool::TunnelPool (int numInboundHops, int numOutboundHops, int numInboundTunnels, int numOutboundTunnels):
+	TunnelPool::TunnelPool (int numInboundHops, int numOutboundHops, int numInboundTunnels,
+		int numOutboundTunnels, int inboundVariance, int outboundVariance):
 		m_NumInboundHops (numInboundHops), m_NumOutboundHops (numOutboundHops),
 		m_NumInboundTunnels (numInboundTunnels), m_NumOutboundTunnels (numOutboundTunnels),
+		m_InboundVariance (inboundVariance), m_OutboundVariance (outboundVariance),
 		m_IsActive (true), m_CustomPeerSelector(nullptr)
 	{
 		if (m_NumInboundTunnels > TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY)
 			m_NumInboundTunnels = TUNNEL_POOL_MAX_INBOUND_TUNNELS_QUANTITY;
 		if (m_NumOutboundTunnels > TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY)
 			m_NumOutboundTunnels = TUNNEL_POOL_MAX_OUTBOUND_TUNNELS_QUANTITY;
+		if (m_InboundVariance < 0 && m_NumInboundHops + m_InboundVariance <= 0)
+			m_InboundVariance = m_NumInboundHops ? -m_NumInboundHops + 1 : 0;
+		if (m_OutboundVariance < 0 && m_NumOutboundHops + m_OutboundVariance <= 0)
+			m_OutboundVariance = m_NumOutboundHops ? -m_NumOutboundHops + 1 : 0;
+		if (m_InboundVariance > 0 && m_NumInboundHops + m_InboundVariance > STANDARD_NUM_RECORDS)
+			m_InboundVariance = (m_NumInboundHops < STANDARD_NUM_RECORDS) ? STANDARD_NUM_RECORDS - m_NumInboundHops : 0;
+		if (m_OutboundVariance > 0 && m_NumOutboundHops + m_OutboundVariance > STANDARD_NUM_RECORDS)
+			m_OutboundVariance = (m_NumOutboundHops < STANDARD_NUM_RECORDS) ? STANDARD_NUM_RECORDS - m_NumOutboundHops : 0;
 		m_NextManageTime = i2p::util::GetSecondsSinceEpoch () + rand () % TUNNEL_POOL_MANAGE_INTERVAL;
 	}
 
@@ -66,12 +76,12 @@ namespace tunnel
 			if (m_NumInboundHops > size)
 			{
 				m_NumInboundHops = size;
-				LogPrint (eLogInfo, "Tunnels: Inbound tunnel length has beed adjusted to ", size, " for explicit peers");
+				LogPrint (eLogInfo, "Tunnels: Inbound tunnel length has been adjusted to ", size, " for explicit peers");
 			}
 			if (m_NumOutboundHops > size)
 			{
 				m_NumOutboundHops = size;
-				LogPrint (eLogInfo, "Tunnels: Outbound tunnel length has beed adjusted to ", size, " for explicit peers");
+				LogPrint (eLogInfo, "Tunnels: Outbound tunnel length has been adjusted to ", size, " for explicit peers");
 			}
 			m_NumInboundTunnels = 1;
 			m_NumOutboundTunnels = 1;
@@ -217,7 +227,7 @@ namespace tunnel
 			if (it->IsEstablished () && it != excluded && (compatible & it->GetFarEndTransports ()))
 			{
 				if (it->IsSlow () || (HasLatencyRequirement() && it->LatencyIsKnown() &&
-				    !it->LatencyFitsRange(m_MinLatency, m_MaxLatency)))
+					!it->LatencyFitsRange(m_MinLatency, m_MaxLatency)))
 				{
 					i++; skipped = true;
 					continue;
@@ -272,8 +282,13 @@ namespace tunnel
 			for (const auto& it : m_OutboundTunnels)
 				if (it->IsEstablished ()) num++;
 		}
-		for (int i = num; i < m_NumOutboundTunnels; i++)
-			CreateOutboundTunnel ();
+		num = m_NumOutboundTunnels - num;
+		if (num > 0)
+		{
+			if (num > TUNNEL_POOL_MAX_NUM_BUILD_REQUESTS) num = TUNNEL_POOL_MAX_NUM_BUILD_REQUESTS;
+			for (int i = 0; i < num; i++)
+				CreateOutboundTunnel ();
+		}
 
 		num = 0;
 		{
@@ -290,8 +305,13 @@ namespace tunnel
 				if (num >= m_NumInboundTunnels) break;
 			}
 		}
-		for (int i = num; i < m_NumInboundTunnels; i++)
-			CreateInboundTunnel ();
+		num = m_NumInboundTunnels - num;
+		if (num > 0)
+		{
+			if (num > TUNNEL_POOL_MAX_NUM_BUILD_REQUESTS) num = TUNNEL_POOL_MAX_NUM_BUILD_REQUESTS;
+			for (int i = 0; i < num; i++)
+				CreateInboundTunnel ();
+		}
 
 		if (num < m_NumInboundTunnels && m_NumInboundHops <= 0 && m_LocalDestination) // zero hops IB
 			m_LocalDestination->SetLeaseSetUpdated (); // update LeaseSet immediately
@@ -372,7 +392,7 @@ namespace tunnel
 
 	void TunnelPool::ManageTunnels (uint64_t ts)
 	{
-		if (ts > m_NextManageTime)
+		if (ts > m_NextManageTime || ts + 2*TUNNEL_POOL_MANAGE_INTERVAL < m_NextManageTime) // in case if clock was adjusted
 		{
 			CreateTunnels ();
 			TestTunnels ();
@@ -411,13 +431,18 @@ namespace tunnel
 		{
 			uint64_t dlt = i2p::util::GetMillisecondsSinceEpoch () - timestamp;
 			LogPrint (eLogDebug, "Tunnels: Test of ", msgID, " successful. ", dlt, " milliseconds");
-			uint64_t latency = dlt / 2;
+			int numHops = 0;
+			if (test.first) numHops += test.first->GetNumHops ();
+			if (test.second) numHops += test.second->GetNumHops ();
 			// restore from test failed state if any
 			if (test.first)
 			{
 				if (test.first->GetState () == eTunnelStateTestFailed)
 					test.first->SetState (eTunnelStateEstablished);
 				// update latency
+				uint64_t latency = 0;
+				if (numHops) latency = dlt*test.first->GetNumHops ()/numHops;
+				if (!latency) latency = dlt/2;
 				test.first->AddLatencySample(latency);
 			}
 			if (test.second)
@@ -425,6 +450,9 @@ namespace tunnel
 				if (test.second->GetState () == eTunnelStateTestFailed)
 					test.second->SetState (eTunnelStateEstablished);
 				// update latency
+				uint64_t latency = 0;
+				if (numHops) latency = dlt*test.second->GetNumHops ()/numHops;
+				if (!latency) latency = dlt/2;
 				test.second->AddLatencySample(latency);
 			}
 		}
@@ -493,7 +521,7 @@ namespace tunnel
 				return false;
 			}
 			if ((i == numHops - 1) && (!hop->IsV4 () || // doesn't support ipv4
-				(inbound && !hop->IsReachable ())))  // IBGW is not reachable
+				(inbound && !hop->IsReachable ()))) // IBGW is not reachable
 			{
 				auto hop1 = nextHop (prevHop, true);
 				if (hop1) hop = hop1;
@@ -507,7 +535,30 @@ namespace tunnel
 
 	bool TunnelPool::SelectPeers (Path& path, bool isInbound)
 	{
-		int numHops = isInbound ? m_NumInboundHops : m_NumOutboundHops;
+		// explicit peers in use
+		if (m_ExplicitPeers) return SelectExplicitPeers (path, isInbound);
+		// calculate num hops
+		int numHops;
+		if (isInbound)
+		{
+			numHops = m_NumInboundHops;
+			if (m_InboundVariance)
+			{
+				int offset = rand () % (std::abs (m_InboundVariance) + 1);
+				if (m_InboundVariance < 0) offset = -offset;
+				numHops += offset;
+			}
+		}
+		else
+		{
+			numHops = m_NumOutboundHops;
+			if (m_OutboundVariance)
+			{
+				int offset = rand () % (std::abs (m_OutboundVariance) + 1);
+				if (m_OutboundVariance < 0) offset = -offset;
+				numHops += offset;
+			}
+		}
 		// peers is empty
 		if (numHops <= 0) return true;
 		// custom peer selector in use ?
@@ -516,8 +567,6 @@ namespace tunnel
 			if (m_CustomPeerSelector)
 				return m_CustomPeerSelector->SelectPeers(path, numHops, isInbound);
 		}
-		// explicit peers in use
-		if (m_ExplicitPeers) return SelectExplicitPeers (path, isInbound);
 		return StandardSelectPeers(path, numHops, isInbound, std::bind(&TunnelPool::SelectNextHop, this, std::placeholders::_1, std::placeholders::_2));
 	}
 
@@ -676,7 +725,7 @@ namespace tunnel
 		auto tunnel = tunnels.CreateInboundTunnel (
 			m_NumOutboundHops > 0 ? std::make_shared<TunnelConfig>(outboundTunnel->GetInvertedPeers (),
 				outboundTunnel->IsShortBuildMessage ()) : nullptr,
-		    shared_from_this (), outboundTunnel);
+				shared_from_this (), outboundTunnel);
 		if (tunnel->IsEstablished ()) // zero hops
 			TunnelCreated (tunnel);
 	}
