@@ -36,12 +36,14 @@ namespace proxy {
 		"reg.i2p",
 		"stats.i2p",
 		"identiguy.i2p",
+		"notbob.i2p"
 	};
 
 	static const std::map<std::string, std::string> jumpservices = {
 		{ "reg.i2p",       "http://shx5vqsw7usdaunyzr2qmes2fq37oumybpudrd4jjj4e4vk4uusa.b32.i2p/jump/" },
 		{ "identiguy.i2p", "http://3mzmrus2oron5fxptw7hw2puho3bnqmw2hqy7nw64dsrrjwdilva.b32.i2p/cgi-bin/query?hostname=" },
 		{ "stats.i2p",     "http://7tbay5p4kzeekxvyvbf6v7eauazemsnnl2aoyqhg5jzpr5eke7tq.b32.i2p/cgi-bin/jump.cgi?a=" },
+		{ "notbob.i2p",    "http://nytzrhrjjfsutowojvxi7hphesskpqqr65wpistz6wa7cpajhp7a.b32.i2p/cgi-bin/jump.cgi?q=" }
 	};
 
 	static const char *pageHead =
@@ -80,8 +82,9 @@ namespace proxy {
 			/* error helpers */
 			void GenericProxyError(const std::string& title, const std::string& description);
 			void GenericProxyInfo(const std::string& title, const std::string& description);
-			void HostNotFound(std::string & host);
-			void SendProxyError(std::string & content);
+			void HostNotFound(std::string& host);
+			void SendProxyError(std::string& content);
+			void SendRedirect(std::string& address);
 
 			void ForwardToUpstreamProxy();
 			void HandleUpstreamHTTPProxyConnect(const boost::system::error_code & ec);
@@ -174,7 +177,7 @@ namespace proxy {
 		SendProxyError(content);
 	}
 
-	void HTTPReqHandler::HostNotFound(std::string & host) {
+	void HTTPReqHandler::HostNotFound(std::string& host) {
 		std::stringstream ss;
 		ss << "<h1>" << tr("Proxy error: Host not found") << "</h1>\r\n"
 		   << "<p>" << tr("Remote host not found in router's addressbook") << "</p>\r\n"
@@ -191,7 +194,7 @@ namespace proxy {
 		SendProxyError(content);
 	}
 
-	void HTTPReqHandler::SendProxyError(std::string & content)
+	void HTTPReqHandler::SendProxyError(std::string& content)
 	{
 		i2p::http::HTTPRes res;
 		res.code = 500;
@@ -207,6 +210,17 @@ namespace proxy {
 			std::bind(&HTTPReqHandler::SentHTTPFailed, shared_from_this(), std::placeholders::_1));
 	}
 
+	void HTTPReqHandler::SendRedirect(std::string& address)
+	{
+		i2p::http::HTTPRes res;
+		res.code = 302;
+		res.add_header("Location", address);
+		res.add_header("Connection", "close");
+		std::string response = res.to_string();
+		boost::asio::async_write(*m_sock, boost::asio::buffer(response), boost::asio::transfer_all(),
+			std::bind(&HTTPReqHandler::SentHTTPFailed, shared_from_this(), std::placeholders::_1));
+	}
+
 	bool HTTPReqHandler::ExtractAddressHelper(i2p::http::URL & url, std::string & b64, bool & confirm)
 	{
 		confirm = false;
@@ -237,6 +251,7 @@ namespace proxy {
 		req.RemoveHeader("Via");
 		req.RemoveHeader("From");
 		req.RemoveHeader("Forwarded");
+		req.RemoveHeader("DNT"); // Useless DoNotTrack flag
 		req.RemoveHeader("Accept", "Accept-Encoding"); // Accept*, but Accept-Encoding
 		/* drop proxy-disclosing headers */
 		req.RemoveHeader("X-Forwarded");
@@ -244,6 +259,18 @@ namespace proxy {
 		/* replace headers */
 		req.UpdateHeader("User-Agent", "MYOB/6.66 (AN/ON)");
 
+		/**
+		 * i2pd PR #1816:
+		 * Android Webview send this with the value set to the application ID, so we drop it,
+		 * but only if it does not belong to an AJAX request (*HttpRequest, like XMLHttpRequest).
+		 */
+		if(req.GetHeader("X-Requested-With") != "") {
+			auto h = req.GetHeader ("X-Requested-With");
+			auto x = h.find("HttpRequest");
+			if (x == std::string::npos) // not found
+				req.RemoveHeader("X-Requested-With");
+		}
+
 		/**
 		 * according to i2p ticket #1862:
 		 * leave Referer if requested URL with same schema, host and port,
@@ -297,7 +324,14 @@ namespace proxy {
 				GenericProxyError(tr("Invalid request"), tr("addresshelper is not supported"));
 				return true;
 			}
-			if (!i2p::client::context.GetAddressBook ().FindAddress (m_RequestURL.host) || m_Confirm)
+
+			if (i2p::client::context.GetAddressBook ().RecordExists (m_RequestURL.host, jump))
+			{
+				std::string full_url = m_RequestURL.to_string();
+				SendRedirect(full_url);
+				return true;
+			}
+			else if (!i2p::client::context.GetAddressBook ().FindAddress (m_RequestURL.host) || m_Confirm)
 			{
 				i2p::client::context.GetAddressBook ().InsertAddress (m_RequestURL.host, jump);
 				LogPrint (eLogInfo, "HTTPProxy: Added address from addresshelper for ", m_RequestURL.host);
@@ -313,7 +347,8 @@ namespace proxy {
 				std::string full_url = m_RequestURL.to_string();
 				std::stringstream ss;
 				ss << tr("Host") << " " << m_RequestURL.host << " <font color=red>" << tr("already in router's addressbook") << "</font>. ";
-				ss << tr(/* tr: The "record" means addressbook's record. That message appears when domain was already added to addressbook, but helper link is opened for it. */ "Click here to update record:" ) << " <a href=\"" << full_url << (full_url.find('?') != std::string::npos ? "&i2paddresshelper=" : "?i2paddresshelper=");
+				ss << tr(/* tr: The "record" means addressbook's record. That message appears when domain was already added to addressbook, but helper link is opened for it. */ "Click here to update record:" );
+				ss << " <a href=\"" << full_url << (full_url.find('?') != std::string::npos ? "&i2paddresshelper=" : "?i2paddresshelper=");
 				ss << jump << "&update=true\">" << tr("Continue") << "</a>.";
 				GenericProxyInfo(tr("Addresshelper found"), ss.str());
 				return true; /* request processed */
